@@ -25,7 +25,7 @@
 #ifndef __LINBOX_lb_blackbox_data_H
 #define __LINBOX_lb_blackbox_data_H
 
-#include "linbox/blackbox/sparse.h"
+#include "linbox/matrix/sparse-matrix.h"
 #include "linbox/util/matrix-stream.h"
 #include <map>
 #include <utility>
@@ -148,7 +148,8 @@ public:
 	{
 		Blackbox<DomainSource> *B_source= static_cast<Blackbox<DomainSource> * >  (ptr);
 		Blackbox<DomainTarget> *B_target = NULL; /*  was not init't */
-		typename Blackbox<DomainSource>::template rebind<DomainTarget>()(*B_target, *B_source, *D);
+//		typename Blackbox<DomainSource>::template rebind<DomainTarget>()(*B_target, *B_source, *D);
+		typename Blackbox<DomainSource>::template rebind<DomainTarget>()(*B_target, *B_source);
 		delete B_source;
 		ptr = B_target;
 	}
