@@ -53,7 +53,7 @@ public:
  * Macro for automatic code generation *
  ***************************************/
 #define LB_BLACKBOX_VISIT(B)				\
-void visit(const B &d){apply(d);}
+void visit(const B &d){this->apply(d);}
 
 
 /**********************************************************************************
