@@ -26,7 +26,7 @@
 #define __LINBOX_lb_domain_data_H
 
 
-#include "linbox-config.h"
+#include "linbox/linbox-config.h"
 #include "linbox/integer.h"
 #include "linbox/field/field-traits.h"
 
