@@ -25,16 +25,13 @@
 #ifndef __LINBOX_lb_domain_type_H
 #define __LINBOX_lb_domain_type_H
 
-#include "linbox/field/modular.h"
-#include "linbox/field/PID-integer.h"
+#include "linbox/ring/modular.h"
+#include "givaro/zring.h"
 #include "linbox/field/gmp-rational.h"
+#include "linbox/field/givaro.h"
 
 #ifdef __LINBOX_HAVE_NTL
-#include "linbox/field/ntl.h"
-#endif
-
-#ifdef __LINBOX_HAVE_GIVARO
-#include "linbox/field/givaro.h"
+#include "linbox/ring/ntl.h"
 #endif
 
 /**************************************
@@ -44,12 +41,12 @@
 #define __LINBOX_MINIMIZE_DOMAIN
 #define __LINBOX_DOMAIN_ONLY
 
-typedef LinBoxTypelist < LinBox::Modular<double>          , LinBoxDumbType> DL1;
-typedef LinBoxTypelist < LinBox::PID_integer              , DL1> DL2;
+typedef LinBoxTypelist < Givaro::Modular<double>          , LinBoxDumbType> DL1;
+typedef LinBoxTypelist < LinBox::Givaro::ZRing<Integer>              , DL1> DL2;
 typedef LinBoxTypelist < LinBox::GMPRationalField         , DL2> DL3;
-typedef LinBoxTypelist < LinBox::Modular<int32_t>         , DL3> DL4;
-typedef LinBoxTypelist < LinBox::Modular<int64_t>         , DL4> DL5;
-typedef LinBoxTypelist < LinBox::Modular<LinBox::integer> , DL5> DL6;
+typedef LinBoxTypelist < Givaro::Modular<int32_t>         , DL3> DL4;
+typedef LinBoxTypelist < LinBox::Givaro::Modular<int64_t>         , DL4> DL5;
+typedef LinBoxTypelist < Givaro::Modular<LinBox::integer> , DL5> DL6;
 
 #ifdef __LINBOX_MINIMIZE_DOMAIN
 typedef DL3 linbox_domain_list;
@@ -71,17 +68,13 @@ typedef DN3 ntl_domain_list;
 typedef LinBoxDumbType ntl_domain_list;
 #endif
 
-#ifdef __LINBOX_HAVE_GIVARO
-typedef LinBoxTypelist < LinBox::GivaroGfq, LinBoxDumbType> DG1;
-typedef LinBoxTypelist < LinBox::GivaroZpz< Givaro::Std32>, DG1> DG2;
+typedef LinBoxTypelist < LinBox::Givaro::GFq, LinBoxDumbType> DG1;
+typedef LinBoxTypelist < Givaro::Modular< int32_t>, DG1> DG2;
 #ifdef __LINBOX_MINIMIZE_DOMAIN
 typedef DG1 givaro_domain_list;
 #else
 typedef DG2 givaro_domain_list;
 #endif
-#else
-typedef LinBoxDumbType givaro_domain_list;
-#endif
 
 // define DomainList to be the list of all domains
 #ifdef __LINBOX_DOMAIN_ONLY
@@ -100,13 +93,13 @@ typedef LinBoxTL::Append< linbox_domain_list, LinBoxTL::Append< ntl_domain_list,
 extern Domain_Factory linbox_domain;
 
 void UpdateDomain(){
-	linbox_domain.add("linbox_field_dbl"      , constructDomain<LinBox::Modular<double> >);
+	linbox_domain.add("linbox_field_dbl"      , constructDomain<Givaro::Modular<double> >);
 	linbox_domain.add("linbox_field_rational" , constructDomain<LinBox::GMPRationalField>);
-	linbox_domain.add("linbox_ring_integer"   , constructDomain<LinBox::PID_integer>);
+	linbox_domain.add("linbox_ring_integer"   , constructDomain<LinBox::Givaro::ZRing<Integer> >);
 #ifndef __LINBOX_MINIMIZE_DOMAIN
-	linbox_domain.add("linbox_field_32"       , constructDomain<LinBox::Modular<int32_t> >);
-	linbox_domain.add("linbox_field_64"       , constructDomain<LinBox::Modular<int64_t> >);
-	linbox_domain.add("linbox_field_mp"       , constructDomain<LinBox::Modular<LinBox::integer> >);
+	linbox_domain.add("linbox_field_32"       , constructDomain<Givaro::Modular<int32_t> >);
+	linbox_domain.add("linbox_field_64"       , constructDomain<LinBox::Givaro::Modular<int64_t> >);
+	linbox_domain.add("linbox_field_mp"       , constructDomain<Givaro::Modular<LinBox::integer> >);
 #endif
 #ifdef __LINBOX_HAVE_NTL
 	linbox_domain.add("ntl_field_ZZ_p"      , constructDomain<LinBox::NTL_ZZ_p>);
@@ -115,11 +108,10 @@ void UpdateDomain(){
 	linbox_domain.add("ntl_ring_integer"    , constructDomain<LinBox::NTL_ZZ>);
 #endif
 #endif
-#ifdef __LINBOX_HAVE_GIVARO
-	linbox_domain.add("givaro_field_gfq"    , constructDomain<LinBox::GivaroGfq>);
+
+	linbox_domain.add("givaro_field_gfq"    , constructDomain<LinBox::Givaro::GFq>);
 #ifndef  __LINBOX_MINIMIZE_DOMAIN
-	linbox_domain.add("givaro_field_32"     , constructDomain<LinBox::GivaroZpz< Givaro::Std32> >);
-#endif
+	linbox_domain.add("givaro_field_32"     , constructDomain<Givaro::Modular< int32_t> >);
 #endif
 }
 
