@@ -4,20 +4,20 @@
  * written
  *  by Bradford Hovinen <hovinen@cis.udel.edu>
  *
- * 
+ *
  * ========LICENCE========
  * This file is part of the library LinBox.
- * 
+ *
  * LinBox is free software: you can redistribute it and/or modify
  * it under the terms of the  GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
- * 
+ *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
@@ -28,7 +28,6 @@
 #ifndef __LINBOX_bbsolve_H
 #define __LINBOX_bbsolve_H
 
-#include <vector>
 #include <algorithm>
 
 // must fix this list...
@@ -112,7 +111,7 @@ namespace LinBox
 		       const Field                     &F,
 		       const WiedemannTraits &traits = WiedemannTraits ())
 	{
-		Vector u;
+		Vector u(A.field());
 		WiedemannSolver<Field> solver (F, traits);
 
 		VectorWrapper::ensureDim (u, A.rowdim ());
@@ -297,11 +296,10 @@ namespace LinBox
 #endif // __LINBOX_bbsolve_H
 
 
-// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,:0,t0,+0,=s
 // Local Variables:
 // mode: C++
 // tab-width: 8
 // indent-tabs-mode: nil
 // c-basic-offset: 8
 // End:
-
+// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,\:0,t0,+0,=s
