@@ -59,6 +59,7 @@ namespace LinBox
 		// of the elements to be reconstructed
 		std::vector< unsigned long >      	randv;
 
+		Integer& result(Integer &d) { std::cout << "should not be called" << std::endl; return d ;} ; // DON'T TOUCH
 	public:
 
 		EarlyMultipCRA(const unsigned long EARLY=DEFAULT_EARLY_TERM_THRESHOLD) :
@@ -75,6 +76,7 @@ namespace LinBox
 			EarlySingleCRA<Domain>::getResidue(m);
 			return m;
 		}
+
 		template<template<class T> class Vect>
 		Vect<Integer>& getResidue(Vect<Integer>& m)
 		{
@@ -82,6 +84,7 @@ namespace LinBox
 			return m;
 		}
 
+		//! Init
 		template<template<class T> class Vect>
 		void initialize (const Integer& D, const Vect<Integer>& e)
 		{
@@ -114,6 +117,26 @@ namespace LinBox
 			FullMultipCRA<Domain>::initialize(D, e);
 		}
 
+		template<class OKDomain>
+		void initialize (const Domain& D, const BlasVector<OKDomain>& e)
+		{
+			// Random coefficients for a linear combination
+			// of the elements to be reconstructed
+			srand48(BaseTimer::seed());
+			randv. resize ( e.size() );
+			for ( std::vector<unsigned long>::iterator int_p = randv. begin();
+			      int_p != randv. end(); ++ int_p)
+				*int_p = ((unsigned long)lrand48()) % 20000;
+			DomainElement z;
+			// Could be much faster
+			// - do not compute twice the product of moduli
+			// - reconstruct one element of e until Early Termination,
+			//   then only, try a random linear combination.
+			EarlySingleCRA<Domain>::initialize(D,dot(z, D, e, randv) );
+			FullMultipCRA<Domain>::initialize(D, e);
+		}
+
+		//! Progress
 		template<template<class T> class Vect>
 		void progress (const Integer& D, const Vect<Integer>& e)
 		{
@@ -123,24 +146,49 @@ namespace LinBox
 			FullMultipCRA<Domain>::progress(D, e);
 		}
 
+#if 1
 		template<template <class> class Alloc, template<class, class> class Vect>
 		void progress (const Domain& D, const Vect<DomainElement, Alloc<DomainElement> >& e)
+		{
+			// DomainElement z;
+			/*!@todo Could be much faster
+			  - do not compute twice the product of moduli
+			  - reconstruct one element of e until Early Termination,
+			  then only, try a random linear combination.
+			*/
+			// EarlySingleCRA<Domain>::progress(D, dot(z, D, e, randv));
+			// FullMultipCRA<Domain>::progress(D, e);
+			BlasVector<Domain> d(D,e);
+			this->progress(D,d);
+		}
+#endif
+
+		template<class OKDomain>
+		void progress (const Domain& D, const BlasVector<OKDomain>& e)
 		{
 			DomainElement z;
-			// Could be much faster
-			// - do not compute twice the product of moduli
-			// - reconstruct one element of e until Early Termination,
-			//   then only, try a random linear combination.
+			/*!@todo Could be much faster
+			  - do not compute twice the product of moduli
+			  - reconstruct one element of e until Early Termination,
+			  then only, try a random linear combination.
+			*/
 			EarlySingleCRA<Domain>::progress(D, dot(z, D, e, randv));
 			FullMultipCRA<Domain>::progress(D, e);
 		}
 
+		//! Result
 		template<template <class> class Alloc, template<class, class> class Vect>
 		Vect<Integer, Alloc<Integer> >& result(Vect<Integer, Alloc<Integer> >& d)
 		{
 			return FullMultipCRA<Domain>::result(d);
 		}
 
+		BlasVector<Givaro::ZRing<Integer> >& result(BlasVector<Givaro::ZRing<Integer> >& d)
+		{
+			return FullMultipCRA<Domain>::result(d);
+		}
+
+		//! terminate
 		bool terminated()
 		{
 			return EarlySingleCRA<Domain>::terminated();
@@ -213,7 +261,7 @@ namespace LinBox
 				    const Vect2& v2)
 		{
 
-			D.init(z,0); DomainElement tmp;
+			D.assign(z,D.zero); DomainElement tmp;
 			typename Vect1<DomainElement, Alloc<DomainElement> >::const_iterator v1_p;
 			typename Vect2::const_iterator v2_p;
 			for (v1_p  = v1. begin(), v2_p = v2. begin();
@@ -225,16 +273,35 @@ namespace LinBox
 			//             commentator().report(Commentator::LEVEL_ALWAYS, INTERNAL_DESCRIPTION) << "z: " << z << std::endl;
 			return z;
 		}
+
+		template <class Vect2, class OKDomain>
+		DomainElement& dot (DomainElement& z, const Domain& D,
+				    const BlasVector<OKDomain>& v1,
+				    const Vect2& v2)
+		{
+
+			D.assign(z,D.zero); DomainElement tmp;
+			typename BlasVector<Domain>::const_iterator v1_p;
+			typename Vect2::const_iterator v2_p;
+			for (v1_p  = v1. begin(), v2_p = v2. begin();
+			     v1_p != v1. end();
+			     ++ v1_p, ++ v2_p)
+				D.axpyin(z, (*v1_p), D.init(tmp, (*v2_p)));
+
+			//             commentator().report(Commentator::LEVEL_ALWAYS, INTERNAL_DESCRIPTION) << "v: " << v2 << std::endl;
+			//             commentator().report(Commentator::LEVEL_ALWAYS, INTERNAL_DESCRIPTION) << "z: " << z << std::endl;
+			return z;
+		}
+
 	};
 }
 #endif //__LINBOX_cra_early_multip_H
 
 
-// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,:0,t0,+0,=s
 // Local Variables:
 // mode: C++
 // tab-width: 8
 // indent-tabs-mode: nil
 // c-basic-offset: 8
 // End:
-
+// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,\:0,t0,+0,=s
