@@ -34,7 +34,7 @@
 #include <stdlib.h>
 #include "linbox/integer.h"
 #include "linbox/solutions/methods.h"
-#include <vector>
+#include "linbox/vector/blas-vector.h"
 #include <utility>
 
 #include "linbox/algorithms/lazy-product.h"
@@ -58,7 +58,7 @@ namespace LinBox
 	protected:
 		std::vector< double >           	RadixSizes_;
 		std::vector< LazyProduct >      	RadixPrimeProd_;
-		std::vector< std::vector<Integer> >    	RadixResidues_;
+		std::vector< BlasVector<Givaro::ZRing<Integer> > >  RadixResidues_;
 		std::vector< bool >             	RadixOccupancy_;
 		const double				LOGARITHMIC_UPPER_BOUND;
 		double					totalsize;
@@ -72,7 +72,8 @@ namespace LinBox
 
 		Integer& getModulus(Integer& m)
 		{
-			std::vector<Integer> r; result(r);
+			Givaro::ZRing<Integer> ZZ;
+			BlasVector<Givaro::ZRing<Integer> > r(ZZ); result(r);
 			return m=RadixPrimeProd_.back()();
 		}
 
@@ -83,25 +84,28 @@ namespace LinBox
 			return r;
 		}
 
+		//! init
 		template<class Vect>
 		void initialize (const Integer& D, const Vect& e)
 		{
 			RadixSizes_.resize(1);
 			RadixPrimeProd_.resize(1);
-			RadixResidues_.resize(1);
+			Givaro::ZRing<Integer> ZZ ;
+			const BlasVector<Givaro::ZRing<Integer> >z(ZZ);
+			RadixResidues_.resize(1,z);
 			RadixOccupancy_.resize(1); RadixOccupancy_.front() = false;
 			progress( D, e);
 #if 0
 			std::vector< double >::iterator  _dsz_it = RadixSizes_.begin();
 			std::vector< LazyProduct >::iterator _mod_it = RadixPrimeProd_.begin();
-			std::vector< std::vector<Integer> >::iterator _tab_it = RadixResidues_.begin();
+			std::vector< BlasVector<Givaro::ZRing<Integer> > >::iterator _tab_it = RadixResidues_.begin();
 			std::vector< bool >::iterator    _occ_it = RadixOccupancy_.begin();
 			_mod_it->initialize(D);
 			*_dsz_it =  Givaro::naturallog(D);
 
 			typename Vect::const_iterator e_it = e.begin();
 			_tab_it->resize(e.size());
-			std::vector<Integer>::iterator t0_it= _tab_it->begin();
+			BlasVector<Givaro::ZRing<Integer> >::iterator t0_it= _tab_it->begin();
 			for( ; e_it != e.end(); ++e_it, ++ t0_it)
 				*t0_it = *e_it;
 			*_occ_it = true;
@@ -114,11 +118,24 @@ namespace LinBox
 		{
 			RadixSizes_.resize(1);
 			RadixPrimeProd_.resize(1);
-			RadixResidues_.resize(1);
+			Givaro::ZRing<Integer> ZZ;
+			RadixResidues_.resize(1,BlasVector<Givaro::ZRing<Integer> >(ZZ));
 			RadixOccupancy_.resize(1); RadixOccupancy_.front() = false;
 			progress(D, e);
 		}
 
+		template<class OKDomain>
+		void initialize (const Domain& D, const BlasVector<OKDomain>& e)
+		{
+			RadixSizes_.resize(1);
+			RadixPrimeProd_.resize(1);
+			Givaro::ZRing<Integer> ZZ ;
+			RadixResidues_.resize(1,BlasVector<Givaro::ZRing<Integer> >(ZZ));
+			RadixOccupancy_.resize(1); RadixOccupancy_.front() = false;
+			progress(D, e);
+		}
+
+		//! progress
 		/* Used in the case where D is a big Integer and Domain cannot be constructed */
 		// template<template<class T> class Vect>
 		template< template<class, class> class Vect, template <class> class Alloc>
@@ -126,13 +143,15 @@ namespace LinBox
 		{
 			std::vector< double >::iterator  _dsz_it = RadixSizes_.begin();
 			std::vector< LazyProduct >::iterator _mod_it = RadixPrimeProd_.begin();
-			std::vector< std::vector<Integer> >::iterator _tab_it = RadixResidues_.begin();
+			std::vector< BlasVector<Givaro::ZRing<Integer> > >::iterator _tab_it = RadixResidues_.begin();
 			std::vector< bool >::iterator    _occ_it = RadixOccupancy_.begin();
-			std::vector<Integer> ri(e.size()); LazyProduct mi; double di;
+			BlasVector<Givaro::ZRing<Integer> > ri(e.field(),e.size());
+			LazyProduct mi;
+			double di;
 			if (*_occ_it) {
 				typename Vect<Integer,Alloc<Integer> >::const_iterator  e_it = e.begin();
-				std::vector<Integer>::iterator       ri_it = ri.begin();
-				std::vector<Integer>::const_iterator t0_it = _tab_it->begin();
+				BlasVector<Givaro::ZRing<Integer> >::iterator       ri_it = ri.begin();
+				BlasVector<Givaro::ZRing<Integer> >::const_iterator t0_it = _tab_it->begin();
 				Integer invprod; precomputeInvProd(invprod, D, _mod_it->operator()());
 				for( ; e_it != e.end(); ++e_it, ++ri_it, ++ t0_it) {
 					*ri_it =* e_it;
@@ -150,7 +169,7 @@ namespace LinBox
 				*_dsz_it = Givaro::naturallog(tmp);
 				typename Vect<Integer, Alloc<Integer> >::const_iterator e_it = e.begin();
 				_tab_it->resize(e.size());
-				std::vector<Integer>::iterator t0_it= _tab_it->begin();
+				BlasVector<Givaro::ZRing<Integer> >::iterator t0_it= _tab_it->begin();
 				for( ; e_it != e.end(); ++e_it, ++ t0_it)
 					*t0_it = *e_it;
 				*_occ_it = true;
@@ -158,8 +177,70 @@ namespace LinBox
 			}
 			for(++_dsz_it, ++_mod_it, ++_tab_it, ++_occ_it ; _occ_it != RadixOccupancy_.end() ; ++_dsz_it, ++_mod_it, ++_tab_it, ++_occ_it) {
 				if (*_occ_it) {
-					std::vector<Integer>::iterator      ri_it = ri.begin();
-					std::vector<Integer>::const_iterator t_it= _tab_it->begin();
+					BlasVector<Givaro::ZRing<Integer> >::iterator      ri_it = ri.begin();
+					BlasVector<Givaro::ZRing<Integer> >::const_iterator t_it= _tab_it->begin();
+					Integer invprod; precomputeInvProd(invprod, mi(), _mod_it->operator()());
+					for( ; ri_it != ri.end(); ++ri_it, ++ t_it)
+						smallbigreconstruct(*ri_it, *t_it, invprod);
+					mi.mulin(*_mod_it);
+					di += *_dsz_it;
+					*_occ_it = false;
+				}
+				else {
+					*_dsz_it = di;
+					*_mod_it = mi;
+					*_tab_it = ri;
+					*_occ_it = true;
+					return;
+				}
+			}
+
+			RadixSizes_.push_back( di );
+			RadixResidues_.push_back( ri );
+			RadixPrimeProd_.push_back( mi );
+			RadixOccupancy_.push_back ( true );
+		}
+
+		// spec for BlasVector
+		void progress (const Integer& D, const BlasVector<Givaro::ZRing<Integer> >& e)
+		{
+			std::vector< double >::iterator  _dsz_it = RadixSizes_.begin();
+			std::vector< LazyProduct >::iterator _mod_it = RadixPrimeProd_.begin();
+			std::vector< BlasVector<Givaro::ZRing<Integer> > >::iterator _tab_it = RadixResidues_.begin();
+			std::vector< bool >::iterator    _occ_it = RadixOccupancy_.begin();
+			BlasVector<Givaro::ZRing<Integer> > ri(e.field(),e.size());
+			LazyProduct mi; double di;
+			if (*_occ_it) {
+				typename BlasVector<Givaro::ZRing<Integer> >::const_iterator  e_it = e.begin();
+				BlasVector<Givaro::ZRing<Integer> >::iterator       ri_it = ri.begin();
+				BlasVector<Givaro::ZRing<Integer> >::const_iterator t0_it = _tab_it->begin();
+				Integer invprod; precomputeInvProd(invprod, D, _mod_it->operator()());
+				for( ; e_it != e.end(); ++e_it, ++ri_it, ++ t0_it) {
+					*ri_it =* e_it;
+					smallbigreconstruct(*ri_it,  *t0_it, invprod );
+				}
+				Integer tmp = D;
+				di = *_dsz_it + Givaro::naturallog(tmp);
+				mi.mulin(tmp);
+				mi.mulin(*_mod_it);
+				*_occ_it = false;
+			}
+			else {
+				Integer tmp = D;
+				_mod_it->initialize(tmp);
+				*_dsz_it = Givaro::naturallog(tmp);
+				typename BlasVector<Givaro::ZRing<Integer> >::const_iterator e_it = e.begin();
+				_tab_it->resize(e.size());
+				BlasVector<Givaro::ZRing<Integer> >::iterator t0_it= _tab_it->begin();
+				for( ; e_it != e.end(); ++e_it, ++ t0_it)
+					*t0_it = *e_it;
+				*_occ_it = true;
+				return;
+			}
+			for(++_dsz_it, ++_mod_it, ++_tab_it, ++_occ_it ; _occ_it != RadixOccupancy_.end() ; ++_dsz_it, ++_mod_it, ++_tab_it, ++_occ_it) {
+				if (*_occ_it) {
+					BlasVector<Givaro::ZRing<Integer> >::iterator      ri_it = ri.begin();
+					BlasVector<Givaro::ZRing<Integer> >::const_iterator t_it= _tab_it->begin();
 					Integer invprod; precomputeInvProd(invprod, mi(), _mod_it->operator()());
 					for( ; ri_it != ri.end(); ++ri_it, ++ t_it)
 						smallbigreconstruct(*ri_it, *t_it, invprod);
@@ -188,16 +269,18 @@ namespace LinBox
 			// Radix shelves
 			std::vector< double >::iterator  _dsz_it = RadixSizes_.begin();
 			std::vector< LazyProduct >::iterator _mod_it = RadixPrimeProd_.begin();
-			std::vector< std::vector<Integer> >::iterator _tab_it = RadixResidues_.begin();
+			std::vector< BlasVector<Givaro::ZRing<Integer> > >::iterator _tab_it = RadixResidues_.begin();
 			std::vector< bool >::iterator    _occ_it = RadixOccupancy_.begin();
-			std::vector<Integer> ri(e.size()); LazyProduct mi; double di;
+			Givaro::ZRing<Integer> ZZ;
+			BlasVector<Givaro::ZRing<Integer> > ri(ZZ,e.size());
+			LazyProduct mi; double di;
 			if (*_occ_it) {
 				// If lower shelf is occupied
 				// Combine it with the new residue
 				// The for loop will try to put the resulting combination on the upper shelf
 				typename Vect<DomainElement, Alloc<DomainElement> >::const_iterator  e_it = e.begin();
-				std::vector<Integer>::iterator       ri_it = ri.begin();
-				std::vector<Integer>::const_iterator t0_it = _tab_it->begin();
+				BlasVector<Givaro::ZRing<Integer> >::iterator       ri_it = ri.begin();
+				BlasVector<Givaro::ZRing<Integer> >::const_iterator t0_it = _tab_it->begin();
 				DomainElement invP0; precomputeInvP0(invP0, D, _mod_it->operator()() );
 				for( ; ri_it != ri.end(); ++e_it, ++ri_it, ++ t0_it)
 					fieldreconstruct(*ri_it, D, *e_it, *t0_it, invP0, (*_mod_it).operator()() );
@@ -219,7 +302,7 @@ namespace LinBox
 				totalsize += ltp;
 				typename Vect<DomainElement, Alloc<DomainElement> >::const_iterator e_it = e.begin();
 				_tab_it->resize(e.size());
-				std::vector<Integer>::iterator t0_it= _tab_it->begin();
+				BlasVector<Givaro::ZRing<Integer> >::iterator t0_it= _tab_it->begin();
 				for( ; e_it != e.end(); ++e_it, ++ t0_it)
 					D.convert(*t0_it, *e_it);
 				*_occ_it = true;
@@ -232,8 +315,8 @@ namespace LinBox
 					// This shelf is occupied
 					// Combine it with the new combination
 					// The loop will try to put it on the upper shelf
-					std::vector<Integer>::iterator      ri_it = ri.begin();
-					std::vector<Integer>::const_iterator t_it= _tab_it->begin();
+					BlasVector<Givaro::ZRing<Integer> >::iterator      ri_it = ri.begin();
+					BlasVector<Givaro::ZRing<Integer> >::const_iterator t_it= _tab_it->begin();
 
 					Integer invprod; precomputeInvProd(invprod, mi(), _mod_it->operator()());
 					for( ; ri_it != ri.end(); ++ri_it, ++ t_it)
@@ -269,12 +352,104 @@ namespace LinBox
 			RadixOccupancy_.push_back ( true );
 		}
 
+		// spec for BlasVector
+		template<class OKDomain>
+		void progress (const Domain& D, const BlasVector<OKDomain >& e)
+		{
+			// Radix shelves
+			std::vector< double >::iterator  _dsz_it = RadixSizes_.begin();
+			std::vector< LazyProduct >::iterator _mod_it = RadixPrimeProd_.begin();
+			std::vector< BlasVector<Givaro::ZRing<Integer> > >::iterator _tab_it = RadixResidues_.begin();
+			std::vector< bool >::iterator    _occ_it = RadixOccupancy_.begin();
+			Givaro::ZRing<Integer> ZZ;
+			LazyProduct mi; double di;
+			BlasVector<Givaro::ZRing<Integer> > ri(ZZ,e.size());
+			if (*_occ_it) {
+				// If lower shelf is occupied
+				// Combine it with the new residue
+				// The for loop will try to put the resulting combination on the upper shelf
+				typename BlasVector<Domain>::const_iterator  e_it = e.begin();
+				BlasVector<Givaro::ZRing<Integer> >::iterator       ri_it = ri.begin();
+				BlasVector<Givaro::ZRing<Integer> >::const_iterator t0_it = _tab_it->begin();
+				DomainElement invP0; precomputeInvP0(invP0, D, _mod_it->operator()() );
+				for( ; ri_it != ri.end(); ++e_it, ++ri_it, ++ t0_it)
+					fieldreconstruct(*ri_it, D, *e_it, *t0_it, invP0, (*_mod_it).operator()() );
+				Integer tmp; D.characteristic(tmp);
+				double ltp = Givaro::naturallog(tmp);
+				di = *_dsz_it + ltp;
+				totalsize += ltp;
+				mi.mulin(tmp);
+				mi.mulin(*_mod_it);
+				*_occ_it = false;
+			}
+			else {
+				// Lower shelf is free
+				// Put the new residue here and exit
+				Integer tmp; D.characteristic(tmp);
+				double ltp =  Givaro::naturallog(tmp);
+				_mod_it->initialize(tmp);
+				*_dsz_it = ltp;
+				totalsize += ltp;
+				typename BlasVector<Domain>::const_iterator e_it = e.begin();
+				_tab_it->resize(e.size());
+				BlasVector<Givaro::ZRing<Integer> >::iterator t0_it= _tab_it->begin();
+				for( ; e_it != e.end(); ++e_it, ++ t0_it)
+					D.convert(*t0_it, *e_it);
+				*_occ_it = true;
+				return;
+			}
+
+			// We have a combination to put in the upper shelf
+			for(++_dsz_it, ++_mod_it, ++_tab_it, ++_occ_it ; _occ_it != RadixOccupancy_.end() ; ++_dsz_it, ++_mod_it, ++_tab_it, ++_occ_it) {
+				if (*_occ_it) {
+					// This shelf is occupied
+					// Combine it with the new combination
+					// The loop will try to put it on the upper shelf
+					BlasVector<Givaro::ZRing<Integer> >::iterator      ri_it = ri.begin();
+					BlasVector<Givaro::ZRing<Integer> >::const_iterator t_it= _tab_it->begin();
+
+					Integer invprod; precomputeInvProd(invprod, mi(), _mod_it->operator()());
+					for( ; ri_it != ri.end(); ++ri_it, ++ t_it)
+						smallbigreconstruct(*ri_it, *t_it, invprod);
+
+					// Product (lazy) computation
+					mi.mulin(*_mod_it);
+
+					// Moding out
+					for(ri_it = ri.begin() ; ri_it != ri.end(); ++ri_it) {
+						*ri_it %= mi();
+					}
+
+					di += *_dsz_it;
+					*_occ_it = false;
+				}
+				else {
+					// This shelf is free
+					// Put the new combination here and exit
+					*_dsz_it = di;
+					*_mod_it = mi;
+					*_tab_it = ri;
+					*_occ_it = true;
+					return;
+				}
+			}
+			// All the shelfves were occupied
+			// We create a new top shelf
+			// And put the new combination there
+			RadixSizes_.push_back( di );
+			RadixResidues_.push_back( ri );
+			RadixPrimeProd_.push_back( mi );
+			RadixOccupancy_.push_back ( true );
+		}
+
+
+		//! result
 		template<template<class, class> class Vect, template <class> class Alloc>
 		Vect<Integer, Alloc<Integer> >& result (Vect<Integer, Alloc<Integer> > &d)
 		{
 			d.resize( (RadixResidues_.front()).size() );
 			std::vector< LazyProduct >::iterator          _mod_it = RadixPrimeProd_.begin();
-			std::vector< std::vector< Integer > >::iterator _tab_it = RadixResidues_.begin();
+			std::vector< BlasVector< Givaro::ZRing<Integer> > >::iterator _tab_it = RadixResidues_.begin();
 			std::vector< bool >::iterator                _occ_it = RadixOccupancy_.begin();
 			LazyProduct Product;
 			// We have to find to lowest occupied shelf
@@ -282,8 +457,8 @@ namespace LinBox
 				if (*_occ_it) {
 					// Found the lowest occupied shelf
 					Product = *_mod_it;
-					std::vector<Integer>::iterator t0_it = d.begin();
-					std::vector<Integer>::iterator t_it = _tab_it->begin();
+					BlasVector<Givaro::ZRing<Integer> >::iterator t0_it = d.begin();
+					BlasVector<Givaro::ZRing<Integer> >::iterator t_it = _tab_it->begin();
 					if (++_occ_it == RadixOccupancy_.end()) {
 						// It is the only shelf of the radix
 						// We normalize the result and output it
@@ -307,8 +482,8 @@ namespace LinBox
 				if (*_occ_it) {
 					// This shelf is occupied
 					// We need to combine it with the actual value of the result
-					std::vector<Integer>::iterator t0_it = d.begin();
-					std::vector<Integer>::const_iterator t_it = _tab_it->begin();
+					BlasVector<Givaro::ZRing<Integer> >::iterator t0_it = d.begin();
+					BlasVector<Givaro::ZRing<Integer> >::const_iterator t_it = _tab_it->begin();
 					Integer invprod;
 					precomputeInvProd(invprod, Product(), _mod_it->operator()() );
 
@@ -337,7 +512,8 @@ namespace LinBox
 			RadixPrimeProd_.front() = Product;
 			RadixSizes_.resize(1);
 			RadixSizes_.front() =  Givaro::naturallog(Product());
-			RadixResidues_.resize(1);
+			Givaro::ZRing<Integer> ZZ;
+			RadixResidues_.resize(1,BlasVector<Givaro::ZRing<Integer> >(ZZ));
 			RadixResidues_.front() = d;
 			RadixOccupancy_.resize(1);
 			RadixOccupancy_.front() = true;
@@ -345,6 +521,84 @@ namespace LinBox
 			return d;
 		}
 
+		// spec for BlasVector
+		BlasVector<Givaro::ZRing<Integer> >& result (BlasVector<Givaro::ZRing<Integer> > &d)
+		{
+			d.resize( (RadixResidues_.front()).size() );
+			std::vector< LazyProduct >::iterator          _mod_it = RadixPrimeProd_.begin();
+			std::vector< BlasVector< Givaro::ZRing<Integer> > >::iterator _tab_it = RadixResidues_.begin();
+			std::vector< bool >::iterator                _occ_it = RadixOccupancy_.begin();
+			LazyProduct Product;
+			// We have to find to lowest occupied shelf
+			for( ; _occ_it != RadixOccupancy_.end() ; ++_mod_it, ++_tab_it, ++_occ_it) {
+				if (*_occ_it) {
+					// Found the lowest occupied shelf
+					Product = *_mod_it;
+					BlasVector<Givaro::ZRing<Integer> >::iterator t0_it = d.begin();
+					BlasVector<Givaro::ZRing<Integer> >::iterator t_it = _tab_it->begin();
+					if (++_occ_it == RadixOccupancy_.end()) {
+						// It is the only shelf of the radix
+						// We normalize the result and output it
+						for( ; t0_it != d.end(); ++t0_it, ++t_it)
+							normalize(*t0_it = *t_it, *t_it, _mod_it->operator()());
+						//RadixPrimeProd_.resize(1);
+						return d;
+					}
+					else {
+						// There are other shelves
+						// The result is initialized with this shelf
+						// The for loop will combine the other shelves m with the actual one
+						for( ; t0_it != d.end(); ++t0_it, ++t_it)
+							*t0_it  = *t_it;
+						++_mod_it; ++_tab_it;
+						break;
+					}
+				}
+			}
+			for( ; _occ_it != RadixOccupancy_.end() ; ++_mod_it, ++_tab_it, ++_occ_it) {
+				if (*_occ_it) {
+					// This shelf is occupied
+					// We need to combine it with the actual value of the result
+					BlasVector<Givaro::ZRing<Integer> >::iterator t0_it = d.begin();
+					BlasVector<Givaro::ZRing<Integer> >::const_iterator t_it = _tab_it->begin();
+					Integer invprod;
+					precomputeInvProd(invprod, Product(), _mod_it->operator()() );
+
+					for( ; t0_it != d.end(); ++t0_it, ++t_it)
+						smallbigreconstruct(*t0_it, *t_it, invprod);
+
+					// Overall product computation
+					Product.mulin(*_mod_it);
+
+					// Moding out and normalization
+					for(t0_it = d.begin();t0_it != d.end(); ++t0_it) {
+						*t0_it %= Product();
+						Integer tmp(*t0_it);
+						normalize(*t0_it, tmp, Product());
+					}
+
+				}
+			}
+
+			// We put it also the final prime product in the first shelf of products
+			// JGD : should we also put the result
+			//       in the first shelf of residues and resize it to 1
+			//       and set to true the first occupancy and resize it to 1
+			//       in case result is not the last call (more progress to go) ?
+			RadixPrimeProd_.resize(1);
+			RadixPrimeProd_.front() = Product;
+			RadixSizes_.resize(1);
+			RadixSizes_.front() =  Givaro::naturallog(Product());
+			Givaro::ZRing<Integer> Z;
+			RadixResidues_.resize(1,BlasVector<Givaro::ZRing<Integer> >(Z));
+			RadixResidues_.front() = d;
+			RadixOccupancy_.resize(1);
+			RadixOccupancy_.front() = true;
+
+			return d;
+		}
+
+
 		bool terminated()
 		{
 			return totalsize > LOGARITHMIC_UPPER_BOUND;
@@ -421,11 +675,10 @@ namespace LinBox
 
 #endif //__LINBOX_cra_full_multip_H
 
-// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,:0,t0,+0,=s
 // Local Variables:
 // mode: C++
 // tab-width: 8
 // indent-tabs-mode: nil
 // c-basic-offset: 8
 // End:
-
+// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,\:0,t0,+0,=s
