@@ -37,6 +37,7 @@
 #include "linbox/vector/vector-domain.h"
 #include "linbox/blackbox/archetype.h"
 #include "linbox/solutions/methods.h"
+#include "linbox/matrix/dense-matrix.h"
 
 
 namespace LinBox
@@ -179,7 +180,8 @@ namespace LinBox
 		*/
 		std::ostream &writePermutation (std::ostream &out, const Permutation &P) const;
 
-	private:
+		const Field & field() { return _MD.field() ; }
+
 		// Compute the kth indexed Gauss-Jordan transform of the input
 		Matrix &kthGaussJordan (unsigned int                  &r,
 					typename Field::Element       &d,
@@ -219,13 +221,12 @@ namespace LinBox
 								 unsigned int dim, const std::vector<unsigned int> &profile);
 
 		// Private variables
+	private:
 
-		const Field                      &_field;
 		VectorDomain<Field>               _VD;
 		MatrixDomain<Field>               _MD;
 		unsigned int                      _number;
 
-		typename Field::Element           _one;
 
 		// Temporaries used in the computation
 
