@@ -29,12 +29,12 @@
 #include "linbox/linbox-config.h"
 #include "linbox/util/debug.h"
 
-#include "linbox/blackbox/sparse.h"
-#include "linbox/field/PID-integer.h"
+#include "linbox/matrix/sparse-matrix.h"
+#include "givaro/zring.h"
 #include "linbox/randiter/random-prime.h"
 #include "linbox/algorithms/rational-solver.h"
 #include "linbox/algorithms/last-invariant-factor.h"
-#include "linbox/field/modular.h"
+#include "linbox/ring/modular.h"
 #include "linbox/algorithms/cra-domain.h"
 #include "linbox/randiter/random-prime.h"
 #include "linbox/algorithms/matrix-hom.h"
@@ -49,19 +49,21 @@ namespace LinBox
 	struct IntegerModularDetReduced {
 	private:
 
-		const Blackbox &A;
-		const MyMethod &M;
+		const                        Blackbox &A;
+		const                        MyMethod &M;
 		/* contains the factor by which to divide det */
-		integer beta;
+		integer                             beta;
 
-		size_t factor;
-		size_t iter_count;
-		size_t iter_count2;
-		typename Vector<PID_integer>::Dense moduli;
+		size_t                            factor;
+		Givaro::ZRing<Integer>                           ZZ; //! @todo is it \c A.field()?
+		size_t                        iter_count;
+		size_t                       iter_count2;
+		typedef  BlasVector<Givaro::ZRing<Integer> >  IVect ;
+		IVect                            moduli ;
 
 	public:
 
-		typename Vector<PID_integer>::Dense primes;
+		IVect                            primes ;
 
 		size_t iterations()
 		{
@@ -74,18 +76,15 @@ namespace LinBox
 		}
 
 
-#if 0
-		int iter_count;
-		int iter_count2;
-		Vector <PID_integer>:: Dense moduli;
-		Vector <PID_integer>:: Dense primes;
-#endif
-
-		IntegerModularDetReduced(const Blackbox& b, const MyMethod& n, const integer& divisor, const size_t& f) :
-			A(b), M(n), beta(divisor), factor(f)
+		IntegerModularDetReduced(const Blackbox& b, const MyMethod& n, const integer& divisor, const size_t& fs) :
+			A(b), M(n)
+			, beta(divisor)
+			, factor(fs)
+			,ZZ(Givaro::ZRing<Integer>())
+			,moduli(ZZ,fs),primes(ZZ,fs)
 		{
-			moduli.resize(factor);
-			primes.resize(factor);
+			// moduli.resize(factor);
+			// primes.resize(factor);
 			iter_count = 0;
 			iter_count2 = 0;
 
@@ -155,7 +154,7 @@ namespace LinBox
 							 const MyMethod                            &M)
 	{
 		//commentator().setReportStream(std::cout);
-		typedef Modular<double> myModular;
+		typedef Givaro::Modular<double> mymodular;
 		typedef typename Blackbox::Field Integers;
 		typedef typename Integers::Element Integer_t;
 
@@ -174,7 +173,7 @@ namespace LinBox
 
 		RandomPrimeIterator genprime( (Integer_t)p_size );
 		//cout << "prime size: " << p_size << "\n";
-		EarlySingleCRA<myModular> cra(4UL);
+		EarlySingleCRA<mymodular> cra(4UL);
 		IntegerModularDetReduced<Blackbox,MyMethod> iteration(A, M, beta,myfactor);
 
 #if 0
@@ -195,10 +194,10 @@ namespace LinBox
 #endif
 		{
 			++genprime;
-			myModular D(*genprime);
+			mymodular D(*genprime);
 			iteration.primes[early_counter] = *genprime;
-			myModular::Element r;
-			D.init(r,0);
+			mymodular::Element r;
+			D.assign(r,D.zero);
 			cra.initialize( D, iteration(r, D));
 			++early_counter;
 		}
@@ -206,11 +205,11 @@ namespace LinBox
 		while ( early_counter < myfactor && !cra.terminated() ) {
 			++genprime;
 			while (cra.noncoprime(*genprime)) ++genprime;
-			myModular D(*genprime);
+			mymodular D(*genprime);
 			iteration.primes[early_counter] = *genprime;
 			// prime(p, early_counter);
-			myModular::Element r;
-			D.init(r,0);
+			mymodular::Element r;
+			D.assign(r,D.zero);
 			cra.progress( D, iteration(r, D));
 			++early_counter;
 		}
@@ -236,11 +235,11 @@ namespace LinBox
 		if (A.rowdim() < 50 ) {
 			while ( !cra.terminated() ) {
 				genprime.randomPrime(p);
-				myModular D(p);
+				mymodular D(p);
 				iteration.primes[early_counter] = p;
 				//		prime(p, early_counter);
-				myModular::Element r;
-				D.init(r,0);
+				mymodular::Element r;
+				D.assign(r,D.zero);
 				cra.progress( D, iteration(r, D));
 				++early_counter;
 			}
@@ -252,13 +251,13 @@ namespace LinBox
 
 		RandomPrime genprime1( 26-(int)ceil(log((double)A.rowdim())*0.7213475205));
 		Integers ZZ;
-		RationalSolver < Integers , myModular, RandomPrime, DixonTraits > RSolver(A. field(), genprime);
+		RationalSolver < Integers , mymodular, RandomPrime, DixonTraits > RSolver(A. field(), genprime);
 #endif
-		RationalSolver < Integers , myModular, RandomPrimeIterator, DixonTraits > RSolver;
+		RationalSolver < Integers , mymodular, RandomPrimeIterator, DixonTraits > RSolver;
 
-		typename Vector<Integers>:: Dense r_num1 (A. coldim());
+		BlasVector<Integers> r_num1 (A.field(),A. coldim());
 
-		LastInvariantFactor < Integers ,RationalSolver < Integers, myModular, RandomPrimeIterator, DixonTraits > >  LIF(RSolver);
+		LastInvariantFactor < Integers ,RationalSolver < Integers, mymodular, RandomPrimeIterator, DixonTraits > >  LIF(RSolver);
 #ifdef _LB_H_DET_TIMING
 		BT.start();
 #endif
@@ -297,14 +296,14 @@ namespace LinBox
 		iteration.Beta(beta);
 
 		//RandomPrime genprime2( 26-(int)ceil(log((double)A.rowdim())*0.7213475205));
-		EarlySingleCRA< Modular<double> > cra2(4UL);
+		EarlySingleCRA< Givaro::Modular<double> > cra2(4UL);
 		Integer_t k = 1;
 
 		early_counter = 0;
 		while ( early_counter < myfactor && !cra2.terminated() ) {
-			myModular D(iteration.primes[early_counter]);
-			myModular::Element r;
-			D.init(r,0);
+			mymodular D(iteration.primes[early_counter]);
+			mymodular::Element r;
+			D.assign(r,D.zero);
 			cra2.progress( D, iteration(r, D) );
 			++early_counter;
 		}
@@ -318,7 +317,7 @@ namespace LinBox
 
 		}
 		else  if (0/* time2 < a*log2(lif)*time1/p_size*/) {
-			typename Vector<Integers>:: Dense r_num2 (A. coldim());
+			BlasVector<Integers> r_num2 (A.field(),A. coldim());
 			Integer_t lif2=1;
 			LIF.lastInvariantFactor1(lif2,r_num2,A);
 			LIF.bonus(bonus,lif, lif2, r_num1,r_num2);
@@ -334,13 +333,13 @@ namespace LinBox
 				//iteration.Moduli(moduli);
 				//iteration.Primes(primes);
 				k=1;
-				EarlySingleCRA< Modular<double> > cra3(4UL);
+				EarlySingleCRA< Givaro::Modular<double> > cra3(4UL);
 
 				early_counter = 0;
 				while ( (early_counter < myfactor) && (!cra3.terminated() )) {
-					myModular D(iteration.primes[early_counter]);
-					myModular::Element r;
-					D.init(r,0);
+					mymodular D(iteration.primes[early_counter]);
+					mymodular::Element r;
+					D.assign(r,D.zero);
 					cra3.progress( D, iteration(r, D));
 					++early_counter;
 					//iteration.Inc();
@@ -361,9 +360,9 @@ namespace LinBox
 					while (!cra3.terminated()) {
 						++genprime;
 						while (cra3.noncoprime(*genprime)) ++genprime;
-						myModular D(*genprime);
-						myModular::Element r;
-						D.init(r,0);
+						mymodular D(*genprime);
+						mymodular::Element r;
+						D.assign(r,D.zero);
 						cra3.progress( D, iteration(r, D));
 					}
 					cra3.result(k);
@@ -378,9 +377,9 @@ namespace LinBox
 				while (!cra2.terminated()) {
 					++genprime;
 					while (cra2.noncoprime(*genprime)) ++genprime;
-					myModular D(*genprime);
-					myModular::Element r;
-					D.init(r,0);
+					mymodular D(*genprime);
+					mymodular::Element r;
+					D.assign(r,D.zero);
 					cra2.progress( D, iteration(r, D));
 				}
 				cra2.result(k);
@@ -393,9 +392,9 @@ namespace LinBox
 			while (!cra2.terminated()) {
 				++genprime;
 				while (cra2.noncoprime(*genprime)) ++genprime;
-				myModular D(*genprime);
-				myModular::Element r;
-				D.init(r,0);
+				mymodular D(*genprime);
+				mymodular::Element r;
+				D.assign(r,D.zero);
 				cra2.progress( D, iteration(r, D));
 			}
 			cra2.result(k);
@@ -437,8 +436,8 @@ namespace LinBox
 	{
 
 		//commentator().setReportStream(std::cout);
-		typedef Modular<double> myModular;
-		//typedef PID_integer Integers;
+		typedef Givaro::Modular<double> mymodular;
+		//typedef Givaro::ZRing<Integer> Integers;
 		typedef typename Integers::Element Integer;
 
 		commentator().start ("Integer Determinant - hybrid version for sparse matrices", "det");
@@ -456,7 +455,7 @@ namespace LinBox
 
 		RandomPrime genprime( (Integer)p_size );
 		commentator().report (Commentator::LEVEL_NORMAL, INTERNAL_DESCRIPTION) << "prime size: " << p_size << "\n";
-		ChineseRemainder< myModular > cra(3UL);
+		ChineseRemainder< mymodular > cra(3UL);
 		IntegerModularDetReduced<SparseMatrix<Integers >,MyMethod> iteration(A, M, beta,myfactor);
 #if 0
 		if (A.rowdim() < 200 ) {
@@ -475,21 +474,21 @@ namespace LinBox
 		BT.start();
 
                 genprime.randomPrime(p);
-                myModular D(p);
+                mymodular D(p);
                 iteration.primes[early_counter] = p;
-                myModular::Element r;
-                D.init(r,0);
+                mymodular::Element r;
+                D.assign(r,D.zero);
                 cra.initialize( D, iteration(r, D));
                 ++early_counter;
 
 		while ( early_counter < myfactor && !cra.terminated() ) {
 			genprime.randomPrime(p);
 			while (cra.noncoprime(p)) genprime.randomPrime(p);
-			myModular D(p);
+			mymodular D(p);
 			iteration.primes[early_counter] = p;
 			//          prime(p, early_counter);
-			myModular::Element r;
-			D.init(r,0);
+			mymodular::Element r;
+			D.assign(r,D.zero);
 			cra.progress( D, iteration(r, D));
 			++early_counter;
 		}
@@ -512,11 +511,11 @@ namespace LinBox
 		if (A.rowdim() < 50 ) {
 			while ( !cra.terminated() ) {
 				genprime.randomPrime(p);
-				myModular D(p);
+				mymodular D(p);
 				iteration.primes[early_counter] = p;
 				//          prime(p, early_counter);
-				myModular::Element r;
-				D.init(r,0);
+				mymodular::Element r;
+				D.assign(r,D.zero);
 				cra.progress( D, iteration(r, D));
 				++early_counter;
 			}
@@ -529,7 +528,7 @@ namespace LinBox
 		RandomPrime genprime1( 26-(int)ceil(log((double)A.rowdim())*0.7213475205));
 		Integers ZZ;
 #endif
-		typedef RationalSolver < Integers , myModular, RandomPrime, BlockHankelTraits > Solver;
+		typedef RationalSolver < Integers , mymodular, RandomPrime, BlockHankelTraits > Solver;
 		Solver RSolver(A. field(), genprime);
 
 		typename Vector<Integers>:: Dense r_num1 (A. coldim());
@@ -568,14 +567,14 @@ namespace LinBox
 		iteration.Beta(beta);
 
 		//RandomPrime genprime2( 26-(int)ceil(log((double)A.rowdim())*0.7213475205));
-		ChineseRemainder< Modular<double> > cra2(3UL);
+		ChineseRemainder< Givaro::Modular<double> > cra2(3UL);
 		Integer k = 1;
 
 		early_counter = 0;
 		while ( early_counter < myfactor && !cra2.terminated() ) {
-			myModular D(iteration.primes[early_counter]);
-			myModular::Element r;
-			D.init(r,0);
+			mymodular D(iteration.primes[early_counter]);
+			mymodular::Element r;
+			D.assign(r,D.zero);
 			cra2.progress( D, iteration(r, D));
 			++early_counter;
 		}
@@ -605,13 +604,13 @@ namespace LinBox
 				//iteration.Moduli(moduli);
 				//iteration.Primes(primes);
 				k=1;
-				ChineseRemainder< Modular<double> > cra3(3UL);
+				ChineseRemainder< Givaro::Modular<double> > cra3(3UL);
 
 				early_counter = 0;
 				while ( (early_counter < myfactor) && (!cra3.terminated() )) {
-					myModular D(iteration.primes[early_counter]);
-					myModular::Element r;
-					D.init(r,0);
+					mymodular D(iteration.primes[early_counter]);
+					mymodular::Element r;
+					D.assign(r,D.zero);
 					cra3.progress( D, iteration(r, D));
 					++early_counter;
 					//iteration.Inc();
@@ -632,9 +631,9 @@ namespace LinBox
 					while (!cra3.terminated()) {
 						genprime.randomPrime(p);
 						while (cra3.noncoprime(p)) genprime.randomPrime(p);
-						myModular D(p);
-						myModular::Element r;
-						D.init(r,0);
+						mymodular D(p);
+						mymodular::Element r;
+						D.assign(r,D.zero);
 						cra3.progress( D, iteration(r, D));
 					}
 					cra3.result(k);
@@ -649,9 +648,9 @@ namespace LinBox
 				while (!cra2.terminated()) {
 					genprime.randomPrime(p);
 					while (cra2.noncoprime(p)) genprime.randomPrime(p);
-					myModular D(p);
-					myModular::Element r;
-					D.init(r,0);
+					mymodular D(p);
+					mymodular::Element r;
+					D.assign(r,D.zero);
 					cra2.progress( D, iteration(r, D));
 				}
 				cra2.result(k);
@@ -664,9 +663,9 @@ namespace LinBox
 			while (!cra2.terminated()) {
 				genprime.randomPrime(p);
 				while (cra2.noncoprime(p)) genprime.randomPrime(p);
-				myModular D(p);
-				myModular::Element r;
-				D.init(r,0);
+				mymodular D(p);
+				mymodular::Element r;
+				D.assign(r,D.zero);
 				cra2.progress( D, iteration(r, D));
 			}
 			cra2.result(k);
@@ -709,11 +708,10 @@ namespace LinBox
 
 
 
-// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,:0,t0,+0,=s
 // Local Variables:
 // mode: C++
 // tab-width: 8
 // indent-tabs-mode: nil
 // c-basic-offset: 8
 // End:
-
+// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,\:0,t0,+0,=s
