@@ -31,10 +31,10 @@
 
 #include "linbox/util/commentator.h"
 #include "linbox/util/timer.h"
-#include "linbox/algorithms/blas-domain.h"
-#include "linbox/field/unparametric.h"
 #include "linbox/matrix/matrix-domain.h"
-#include "linbox/matrix/blas-matrix.h"
+#include <givaro/zring.h>
+#include "linbox/matrix/matrix-domain.h"
+#include "linbox/matrix/dense-matrix.h"
 #include "linbox/matrix/factorized-matrix.h"
 
 #include "linbox/algorithms/matpoly-mult.h"
@@ -48,6 +48,7 @@
 /*! @file algorithms/sigma-basis.h
  * @brief \f$\sigma\f$-basis (minimal basis).
  */
+#define MBASIS_THRESHOLD 16
 
 namespace LinBox
 {
@@ -67,8 +68,9 @@ namespace LinBox
 		typedef typename Field::Element        Element;
 		typedef BlasMatrix<Field>          Coefficient;
 
+		inline const Field & field() const { return *_field; }
 	private:
-		Field                             _field;
+		const Field                    * _field;
 		BlasMatrixDomain<Field>         _BMD;
 		MatrixDomain<Field>              _MD;
 		std::vector<Coefficient>     &_Serie;
@@ -134,7 +136,7 @@ namespace LinBox
 	public:
 
 		SigmaBasis(const Field &F, std::vector<Coefficient> &PowerSerie) :
-			_field(F), _BMD(F), _MD(F), _Serie(PowerSerie), PM_domain(F)
+			_field(&F), _BMD(F), _MD(F), _Serie(PowerSerie), PM_domain(F)
 		{
 #ifdef  _BM_TIMING
 			clearTimer();
@@ -151,7 +153,7 @@ namespace LinBox
 			size_t m = _Serie[0].rowdim();
 			size_t n = _Serie[0].coldim();
 
-			const Coefficient TransposedZero(n,m);
+			const Coefficient TransposedZero(field(),n,m);
 
 			// take the transposed of the Serie and use PM_Basis
 			std::vector<Coefficient> TransposedSerie (length, TransposedZero);
@@ -161,7 +163,7 @@ namespace LinBox
 					for (size_t j=0;j<n;++j)
 						TransposedSerie[k].setEntry(j,i, _Serie[k].getEntry(i,j));
 
-			const Coefficient Zero(n, n);
+			const Coefficient Zero(field(),n, n);
 
 			// Prepare SigmaBase
 			SigmaBase.resize(degree+1, Zero);
@@ -176,7 +178,7 @@ namespace LinBox
 		{
 
 			const size_t m = _Serie[0].rowdim();
-			const Coefficient Zero(m, m);
+			const Coefficient Zero(field(), m, m);
 
 			// Prepare SigmaBase
 			SigmaBase.resize(degree+1, Zero);
@@ -198,8 +200,8 @@ namespace LinBox
 
 			const size_t m = _Serie[0].rowdim();
 			const size_t n = _Serie[0].coldim();
-			const Coefficient Zero(m, m);
-			const Coefficient ZeroSerie(m,n);
+			const Coefficient Zero(field(),m, m);
+			const Coefficient ZeroSerie(field(),m,n);
 
 
 			// Prepare SigmaBases
@@ -238,8 +240,8 @@ namespace LinBox
 			size_t length=_Serie.size();
 			size_t m = _Serie[0].rowdim();
 			size_t n = _Serie[0].coldim();
-			const Coefficient TransposedZero(n,m);
-			const Coefficient Zero(n, n);
+			const Coefficient TransposedZero(field(),n,m);
+			const Coefficient Zero(field(),n, n);
 
 			// take the transposed of the Serie and use PM_Basis
 			std::vector<Coefficient> TransposedSerie (length, TransposedZero);
@@ -295,7 +297,7 @@ namespace LinBox
 
 
 			// transpose the PowerSerie
-			const Coefficient Zero(m,n);
+			const Coefficient Zero(field(),m,n);
 			std::vector<Coefficient> TransPowerSerie(deg,Zero);
 			for (size_t i=0;i<deg;++i){
 				for (size_t j=0;j<m;++j)
@@ -344,7 +346,7 @@ namespace LinBox
 
 
 			// transpose the PowerSerie
-			const Coefficient Zero(m,n);
+			const Coefficient Zero(field(),m,n);
 			std::vector<Coefficient> TransPowerSerie(deg,Zero);
 			for (size_t i=0;i<deg;++i){
 				for (size_t j=0;j<m;++j)
@@ -379,25 +381,24 @@ namespace LinBox
 		//
 		// SigmaBase must be already allocated with degree+1 elements
 		// PowerSerie must have at least degree+1 element
-#define MBASIS_THRESHOLD 16
 
 		template <class Polynomial1, class Polynomial2>
-		void PM_Basis(Polynomial1      &SigmaBase,
+		void PM_Basis(Polynomial1            &SigmaBase,
 			      const Polynomial2     &PowerSerie,
-			      size_t               degree,
-			      std::vector<size_t> &defect)
+			      size_t                     degree,
+			      std::vector<size_t>       &defect)
 		{
 
 			size_t m,n;
 			m = PowerSerie[0].rowdim();
 			n = PowerSerie[0].coldim();
-			const Coefficient ZeroSigma(m,m);
-			const Coefficient ZeroSerie(m,n);
+			const Coefficient ZeroSigma(field(),m,m);
+			const Coefficient ZeroSerie(field(),m,n);
 
 			if (degree == 0) {
-				Coefficient Identity(m,m);
+				Coefficient Identity(field(),m,m);
 				for (size_t i=0;i< m;++i)
-					Identity.setEntry(i,i,_field.one);
+					Identity.setEntry(i,i,field().one);
 				SigmaBase[0]=Identity;
 			}
 
@@ -445,7 +446,7 @@ namespace LinBox
 
 					//PM_domain.midproduct(Serie2,Sigma1,PowerSerie);
 					ComputeNewSerie(Serie2,Sigma1,PowerSerie, degree1, degree2);
-					Serie2.resize(degree2+1);
+					Serie2.resize(degree2+1,ZeroSerie);
 
 #ifdef _BM_TIMING
 					tUpdateSerie.stop();ttUpdateSerie += tUpdateSerie;
@@ -477,7 +478,7 @@ namespace LinBox
 		// algorithm is from Giorgi, Jeannerod and Villard  ISSAC'03
 		template <class Polynomial1, class Polynomial2>
 		void M_Basis(Polynomial1        &SigmaBase,
-			     Polynomial2        PowerSerie,
+			     Polynomial2       &PowerSerie,
 			     size_t                 length,
 			     std::vector<size_t>   &defect)
 		{
@@ -489,15 +490,16 @@ namespace LinBox
 			n = PowerSerie[0].coldim();
 
 			// Set some useful constants
-			const Coefficient Zero(m,m);
+			const Coefficient Zero(field(),m,m);
 
 			// Reserve memory for the Sigma Base and set SigmaBase[0] to Identity
 			SigmaBase.reserve(length+1);
-			SigmaBase.resize(1);
-			Coefficient Identity(m,m);
+			SigmaBase.resize(1,Zero);
+			Coefficient Identity(field(),m,m);
 			for (size_t i=0;i< m;++i)
-				Identity.setEntry(i,i,_field.one);
-			SigmaBase[0]=Identity;
+				Identity.setEntry(i,i,field().one);
+			// SigmaBase.resize(1,Identity) won't work
+			SigmaBase[0] = Identity;
 
 			// Keep track on Sigma Base's row degree
 			// I adjust the degree with the maximal difference between defects
@@ -514,7 +516,7 @@ namespace LinBox
 
 
 			// Discrepancy
-			Coefficient Discrepancy(m,n);
+			Coefficient Discrepancy(field(),m,n);
 			Timer chrono;
 #ifdef  _BM_TIMING
 			int cptr=0;
@@ -533,7 +535,7 @@ namespace LinBox
 				for (size_t i=0;i<m;++i) {
 					size_t idx_min=i;
 					for (size_t j=i+1;j<m;++j)
-						if (defect[j]< defect[idx_min])
+						if (defect[(size_t)j]< defect[idx_min])
 							idx_min=j;
 					std::swap(defect[i], defect[idx_min]);
 					Perm1[i]=idx_min;
@@ -562,10 +564,11 @@ namespace LinBox
 				chrono.start();
 #endif
 				// Compute Discrepancy
-				_BMD.mul(Discrepancy,SigmaBase[0],PowerSerie[k]);
+                                _BMD.mul(Discrepancy,SigmaBase[0],PowerSerie[k]);
 				for (size_t i=1;i<SigmaBase.size();++i){
 					_BMD.axpyin(Discrepancy,SigmaBase[i],PowerSerie[k-i]);
 				}
+
 #ifdef  _BM_TIMING
 				cptr+=SigmaBase.size();
 				chrono.stop();
@@ -575,7 +578,7 @@ namespace LinBox
 #endif
 
 				//std::cout<<"MBasis: Discrepancy\n";
-				//Discrepancy.write(std::cout,_field);
+				//Discrepancy.write(std::cout,field());
 
 
 				// Compute LQUP of Discrepancy
@@ -584,8 +587,8 @@ namespace LinBox
 				LQUPMatrix<Field> LQUP(Discrepancy,P,Qt);
 
 				// Get L from LQUP
-				TriangularBlasMatrix<Field> L(_field, m, m,
-							      LinBoxTag::Lower, LinBoxTag::Unit);
+				TriangularBlasMatrix<Field> L(field(), m, m,
+							      Tag::Shape::Lower, Tag::Diag::Unit);
 				LQUP.getL(L);
 
 				// get the transposed permutation of Q from LQUP
@@ -593,9 +596,9 @@ namespace LinBox
 
 
 				// Compute the inverse of L
-				TriangularBlasMatrix<Field> invL(_field, m, m,
-								 LinBoxTag::Lower, LinBoxTag::Unit);
-				FFPACK::trinv_left((typename Field::Father_t)_field,m,L.getPointer(),L.getStride(),invL.getWritePointer(),invL.getStride());
+				TriangularBlasMatrix<Field> invL(field(), m, m,
+								 Tag::Shape::Lower, Tag::Diag::Unit);
+				FFPACK::trinv_left((const Field &)field(),m,L.getPointer(),L.getStride(),invL.getWritePointer(),invL.getStride());
 
 #ifdef  _BM_TIMING
 				chrono.stop();
@@ -640,11 +643,11 @@ namespace LinBox
 					for (int j= (int) size-2;j>=0; --j){
 						for (size_t l=0;l<m;++l)
 							//BB: #warning Q[i] pour i>r ne veut rien dire...
-							_field.assign(SigmaBase[j+1].refEntry(*(Qt.getPointer()+i),l),
-								  SigmaBase[j].getEntry(*(Qt.getPointer()+i),l));
+							field().assign(SigmaBase[(size_t)j+1].refEntry(*(Qt.getPointer()+i),l),
+								  SigmaBase[(size_t)j].getEntry(*(Qt.getPointer()+i),l));
 					}
 					for (size_t l=0;l<m;++l)
-						_field.assign(SigmaBase[0].refEntry(*(Qt.getPointer()+i),l),_field.zero);
+						field().assign(SigmaBase[0].refEntry(*(Qt.getPointer()+i),l),field().zero);
 				}
 #ifdef  _BM_TIMING
 				chrono.stop();
@@ -670,7 +673,7 @@ namespace LinBox
 			// degree1 >= degree2
 			//size_t size = 2*degree1 + 1;
 
-			const Coefficient ZeroSerie (OldSerie[0].rowdim(), OldSerie[0].coldim());
+			const Coefficient ZeroSerie (field(),OldSerie[0].rowdim(), OldSerie[0].coldim());
 			//const Coefficient ZeroBase  (SigmaBase[0].rowdim(), SigmaBase[0].coldim());
 
 			// Work on a copy of the old  Serie (increase size by one for computation of middle product)
@@ -703,7 +706,7 @@ namespace LinBox
 
 			for (size_t i=0;i<degree2+1;++i)
 				for (size_t j=0;j<degree1+1;++j)
-					_BMD.axpyin(NewSerie[i],SigmaBase[j],OldSerie[degree1+i-j]);
+					_BMD.axpyin(NewSerie[i],SigmaBase[(size_t)j],OldSerie[degree1+i-j]);
 		}
 
 
@@ -731,16 +734,16 @@ namespace LinBox
 			// Initialization of the Serie iterator
 			typename std::vector<Coefficient>::const_iterator _iter (PowerSerie.begin ());
 
-			Coefficient Unit(m+n,m);
-			const Coefficient Zero(m+n,m);
+			Coefficient Unit(field(),m+n,m);
+			const Coefficient Zero(field(),m+n,m);
 			for (size_t i=0;i<m;i++)
-				Unit.setEntry(i,i,_field.one);
+				Unit.setEntry(i,i,field().one);
 			size_t min_mn=(m <n)? m :n;
 
 			// initialization of discrepancy
-			Coefficient Discrepancy(m+n,n);
+			Coefficient Discrepancy(field(),m+n,n);
 			for (size_t i=0;i<n;i++)
-				Discrepancy.setEntry(i+m,i,_field.one);
+				Discrepancy.setEntry(i+m,i,field().one);
 
 
 			// initialization of sigma base
@@ -791,7 +794,7 @@ namespace LinBox
 
 				typename Coefficient::Iterator _iter_Discr = Discr.Begin();
 
-				while ((_field.isZero(*_iter_Discr) && _iter_Discr != Discr.End()))
+				while ((field().isZero(*_iter_Discr) && _iter_Discr != Discr.End()))
 					++_iter_Discr;
 
 				// maybe there is something to do here
@@ -813,7 +816,7 @@ namespace LinBox
 					for (size_t i=0;i<m+n;++i) {
 						size_t idx_min=i;
 						for (size_t j=i+1;j<m+n;++j)
-							if (order[j]< order[idx_min])
+							if (order[(size_t)j]< order[idx_min])
 								idx_min=j;
 						std::swap(order[i],order[idx_min]);
 						Perm1[i]=idx_min;
@@ -835,8 +838,8 @@ namespace LinBox
 				LQUPMatrix<Field> LQUP(CopyDiscr,P,Qt);
 
 				// Get the matrix L of LQUP decomposition
-				TriangularBlasMatrix<Field> L(_field,m+n,m+n,
-							      LinBoxTag::Lower, LinBoxTag::Unit );
+				TriangularBlasMatrix<Field> L(field(),m+n,m+n,
+							      Tag::Shape::Lower, Tag::Diag::Unit );
 				LQUP.getL(L);
 
 				// Get the tranposed  permutation of Q from LQUP
@@ -851,9 +854,9 @@ namespace LinBox
 				BlasPermutation<size_t> BPerm2(Perm2);
 
 				// compute the inverse of L
-				TriangularBlasMatrix<Field> invL (_field,m+n,m+n,
-								  LinBoxTag::Lower,LinBoxTag::Unit);
-				FFPACK::trinv_left((typename Field::Father_t)_field,m+n, L.getPointer(), L.getStride(),
+				TriangularBlasMatrix<Field> invL (field(),m+n,m+n,
+								  Tag::Shape::Lower,Tag::Diag::Unit);
+				FFPACK::trinv_left((const Field &)field(),m+n, L.getPointer(), L.getStride(),
 						   invL.getWritePointer(), invL.getStride());
 
 
@@ -867,8 +870,8 @@ namespace LinBox
 
 
 				// Apply BPerm2 and Qt to the vector of order and increase by 1 the last n rows
-				UnparametricField<long> UF;
-				BlasMatrixDomain<UnparametricField<long> > BMDUF(UF);
+				Givaro::ZRing<long> UF;
+				BlasMatrixDomain<Givaro::ZRing<long> > BMDUF(UF);
 				BMDUF.mulin_right(Qt,order);
 				BMDUF.mulin_right(BPerm2,order);
 				BMDUF.mulin_right(BPerm1,degree);
@@ -895,17 +898,17 @@ namespace LinBox
 				for (int i= (int)size-2;i>=0;i--)
 					for (size_t j=0;j<n;j++)
 						for (size_t k=0;k<n;++k)
-							_field.assign(SigmaBase[i+1].refEntry(m+j,k), SigmaBase[i].getEntry(m+j,k));
+							field().assign(SigmaBase[i+1].refEntry(m+j,k), SigmaBase[i].getEntry(m+j,k));
 
 				for (size_t j=0;j<n;j++)
 					for (size_t k=0;k<n;++k)
-						_field.assign(SigmaBase[0].refEntry(m+j,k),_field.zero);
+						field().assign(SigmaBase[0].refEntry(m+j,k),field().zero);
 
 
 				// Discrepancy= BPerm2.U.P from LQUP
-				Coefficient U(m+n,n);
+				Coefficient U(field(),m+n,n);
 				TriangularBlasMatrix<Field> trU( U,
-								LinBoxTag::Upper,LinBoxTag::NonUnit);
+								Tag::Shape::Upper,Tag::Diag::NonUnit);
 				LQUP.getU(trU);
 				Discrepancy=U;
 				// BlasPermutation<size_t> P= LQUP.getP();
@@ -924,13 +927,13 @@ namespace LinBox
 					max=degree[i];
 			}
 
-			const Coefficient AZero(m,m);
+			const Coefficient AZero(field(),m,m);
 			Approx.resize(max+1, AZero);
 
 			for (size_t i=0;i<m;i++)
 				for (long j=0;j<=degree[i];j++)
 					for (size_t k=0;k<m;k++)
-						_field.assign(Approx[j].refEntry(i,k), SigmaBase[j].getEntry(i,k));
+						field().assign(Approx[(size_t)j].refEntry(i,k), SigmaBase[(size_t)j].getEntry(i,k));
 
 
 		}
@@ -954,16 +957,16 @@ namespace LinBox
 			// Initialization of the Serie iterator
 			typename std::vector<Coefficient>::const_iterator _iter (PowerSerie.begin ());
 
-			Coefficient Unit(m+n,m);
-			const Coefficient Zero(m+n,m);
+			Coefficient Unit(field(),m+n,m);
+			const Coefficient Zero(field(),m+n,m);
 			for (size_t i=0;i<m;i++)
-				Unit.setEntry(i,i,_field.one);
+				Unit.setEntry(i,i,field().one);
 			size_t min_mn=(m <n)? m :n;
 
 			// initialization of discrepancy
-			Coefficient Discrepancy(m+n,n);
+			Coefficient Discrepancy(field(),m+n,n);
 			for (size_t i=0;i<n;i++)
-				Discrepancy.setEntry(i+m,i,_field.one);
+				Discrepancy.setEntry(i+m,i,field().one);
 
 
 			// initialization of sigma base
@@ -1014,7 +1017,7 @@ namespace LinBox
 
 				typename Coefficient::Iterator _iter_Discr = Discr.Begin();
 
-				while ((_field.isZero(*_iter_Discr) && _iter_Discr != Discr.End()))
+				while ((field().isZero(*_iter_Discr) && _iter_Discr != Discr.End()))
 					++_iter_Discr;
 
 				// maybe there is something to do here
@@ -1036,7 +1039,7 @@ namespace LinBox
 					for (size_t i=0;i<m+n;++i) {
 						size_t idx_min=i;
 						for (size_t j=i+1;j<m+n;++j)
-							if (order[j]< order[idx_min])
+							if (order[(size_t)j]< order[idx_min])
 								idx_min=j;
 						std::swap(order[i],order[idx_min]);
 						Perm1[i]=idx_min;
@@ -1058,8 +1061,8 @@ namespace LinBox
 				LQUPMatrix<Field> LQUP(CopyDiscr,P,Qt);
 
 				// Get the matrix L of LQUP decomposition
-				TriangularBlasMatrix<Field> L(_field, m+n,m+n,
-							      LinBoxTag::Lower, LinBoxTag::Unit );
+				TriangularBlasMatrix<Field> L(field(), m+n,m+n,
+							      Tag::Shape::Lower, Tag::Diag::Unit );
 				LQUP.getL(L);
 
 				// Get the tranposed  permutation of Q from LQUP
@@ -1074,9 +1077,9 @@ namespace LinBox
 				BlasPermutation<size_t> BPerm2(Perm2);
 
 				// compute the inverse of L
-				TriangularBlasMatrix<Field> invL (_field,m+n,m+n,
-								  LinBoxTag::Lower,LinBoxTag::Unit);
-				FFPACK::trinv_left((typename Field::Father_t)_field,m+n,L.getPointer(),L.getStride(),
+				TriangularBlasMatrix<Field> invL (field(),m+n,m+n,
+								  Tag::Shape::Lower,Tag::Diag::Unit);
+				FFPACK::trinv_left(field(),m+n,L.getPointer(),L.getStride(),
 						   invL.getWritePointer(),invL.getStride());
 
 
@@ -1090,8 +1093,8 @@ namespace LinBox
 
 
 				// Apply BPerm2 and Qt to the vector of order and increase by 1 the last n rows
-				UnparametricField<long> UF;
-				BlasMatrixDomain<UnparametricField<long> > BMDUF(UF);
+				Givaro::ZRing<long> UF;
+				BlasMatrixDomain<Givaro::ZRing<long> > BMDUF(UF);
 				BMDUF.mulin_right(Qt,order);
 				BMDUF.mulin_right(BPerm2,order);
 				BMDUF.mulin_right(BPerm1,degree);
@@ -1118,16 +1121,16 @@ namespace LinBox
 				for (int i= (int)size-2;i>=0;i--)
 					for (size_t j=0;j<n;j++)
 						for (size_t k=0;k<n;++k)
-							_field.assign(SigmaBase[i+1].refEntry(m+j,k), SigmaBase[i].getEntry(m+j,k));
+							field().assign(SigmaBase[i+1].refEntry(m+j,k), SigmaBase[i].getEntry(m+j,k));
 
 				for (size_t j=0;j<n;j++)
 					for (size_t k=0;k<n;++k)
-						_field.assign(SigmaBase[0].refEntry(m+j,k),_field.zero);
+						field().assign(SigmaBase[0].refEntry(m+j,k),field().zero);
 
 
 				// Discrepancy= BPerm2.U.P from LQUP
-				Coefficient U(m+n,n);
-				TriangularBlasMatrix<Field> trU(U,LinBoxTag::Upper,LinBoxTag::NonUnit);
+				Coefficient U(field(),m+n,n);
+				TriangularBlasMatrix<Field> trU(U,Tag::Shape::Upper,Tag::Diag::NonUnit);
 				LQUP.getU(trU);
 				Discrepancy=U;
 				// BlasPermutation<size_t> P= LQUP.getP();
@@ -1143,13 +1146,13 @@ namespace LinBox
 							max=order[i];
 					}
 
-					const Coefficient AZero(m,m);
+					const Coefficient AZero(field(),m,m);
 					Approx1.resize(max+1, AZero);
 
 					for (size_t i=0;i<m;i++)
 						for (long j=0;j<=order[i];j++)
 							for (size_t k=0;k<m;k++)
-								_field.assign(Approx1[j].refEntry(i,k), SigmaBase[j].getEntry(i,k));
+								field().assign(Approx1[(size_t)j].refEntry(i,k), SigmaBase[(size_t)j].getEntry(i,k));
 				}
 
 			}
@@ -1165,13 +1168,13 @@ namespace LinBox
 					max=degree[i];
 			}
 
-			const Coefficient AZero(m,m);
+			const Coefficient AZero(field(),m,m);
 			Approx2.resize(max+1, AZero);
 
 			for (size_t i=0;i<m;i++)
 				for (long j=0;j<=degree[i];j++)
 					for (size_t k=0;k<m;k++)
-						_field.assign(Approx2[j].refEntry(i,k), SigmaBase[j].getEntry(i,k));
+						field().assign(Approx2[(size_t)j].refEntry(i,k), SigmaBase[(size_t)j].getEntry(i,k));
 		}
 
 
@@ -1190,28 +1193,30 @@ namespace LinBox
 			n = PowerSerie[0].coldim();
 
 			// Set some useful constants
-			const Coefficient Zeromm(m,m);
-			const Coefficient Zeromn(m,n);
+			const Coefficient Zeromm(field(),m,m);
+			const Coefficient Zeromn(field(),m,n);
 
 			// Reserve memory for the Sigma Base
 			SigmaBase.reserve(length+1);
 			SigmaBase.resize(1);
 
 			// set SigmaBase[0] to Identity
-			Coefficient Identity(m,m);
+			Coefficient Identity(field(),m,m);
 			for (size_t i=0;i< m;++i)
-				Identity.setEntry(i,i,_field.one);
+				Identity.setEntry(i,i,field().one);
 			SigmaBase[0]=Identity;
 
 			// Define Truncated Residual
 			std::vector<Coefficient>  Residual(length+1, Zeromn);
 
 			// Set Truncated Residual to PowerSerie mod X^length
+#if 0
 			for (size_t k=0;k<length; ++k)
 				Residual[k] = PowerSerie[k];
+#endif
 
 			// Define Discrepancy
-			Coefficient Discrepancy(m,n);
+			Coefficient Discrepancy(field(),m,n);
 
 			// Row degree of SigmaBase
 			// Keep track on Sigma Base's row degree
@@ -1396,7 +1401,7 @@ namespace LinBox
 				for (size_t i=0;i<m;++i) {
 					size_t idx_min=i;
 					for (size_t j=i+1;j<m;++j)
-						if (defect[j]< defect[idx_min])
+						if (defect[(size_t)j]< defect[idx_min])
 							idx_min=j;
 					std::swap(defect[i], defect[idx_min]);
 					Perm1[i]=idx_min;
@@ -1428,7 +1433,7 @@ namespace LinBox
 #endif
 
 				/* new version : use of columnReducedEchelon */
-				EchelonFormDomain<Field>  EFD(_field);
+				EchelonFormDomain<Field>  EFD(field());
 				size_t rank = EFD.columnReducedEchelon(Discrepancy);
 
 
@@ -1438,7 +1443,7 @@ namespace LinBox
 					perm[i]=i;
 				size_t idx=0;
 				for (size_t i=0;i<rank;++i){
-					while(_field.isZero(Discrepancy.getEntry(idx,i))) idx++;
+					while(field().isZero(Discrepancy.getEntry(idx,i))) idx++;
 					perm[i]=idx;
 					idx++;
 				}
@@ -1485,17 +1490,17 @@ namespace LinBox
 					maxs=std::max(maxs, degree[*(Qt.getPointer()+d)]);
 				maxs=std::min(maxs, SigmaBase.size()-1);
 
-				//Discrepancy.write(std::cout,_field);
+				//Discrepancy.write(std::cout,field());
 
 
 #ifndef OPTMIZED_SIGMA_UPDATE
 				// Compute a Permutation to put all trivial columns of SigmaBase to right part of it
 				std::vector<size_t> PermTrivial(m);
-				bool PTrivial=true;
 				for(size_t i=0;i<m;++i)
 					if (PermPivots[i]>i)
 						std::swap(triv_column[i], triv_column[PermPivots[i]]);
 
+				// bool PTrivial=true; // no need for that here
 				if (nbr_triv > (int)rank) {
 					size_t idx_triv, idx_nontriv;
 					idx_nontriv = 0;
@@ -1504,12 +1509,12 @@ namespace LinBox
 					for (size_t i=0;i<m;++i){
 						if (triv_column[i]!=0){
 							PermTrivial[i]=idx_nontriv;
-							if (i!=idx_nontriv) PTrivial=false;
+							// if (i!=idx_nontriv) PTrivial=false;
 							idx_nontriv++;
 						}
 						else {
 							PermTrivial[i]=idx_triv;
-							if (i!=idx_triv) PTrivial=false;
+							// if (i!=idx_triv) PTrivial=false;
 							idx_triv++;
 						}
 					}
@@ -1549,7 +1554,7 @@ namespace LinBox
 				std::cout<<"\n";
 
 				write_maple("Sigma",SigmaBase);
-				//Discrepancy.write(std::cout,_field);
+				//Discrepancy.write(std::cout,field());
 
 
 
@@ -1558,13 +1563,13 @@ namespace LinBox
 					std::cout<<triv_column[i]<<", ";
 				std::cout<<"\n";
 
-				SigmaBase[0].write(std::cout,_field);
+				SigmaBase[0].write(std::cout,field());
 				_BMD.mulin_right(BPerm1, SigmaBase[0]);
-				SigmaBase[0].write(std::cout,_field);
+				SigmaBase[0].write(std::cout,field());
 				_BMD.mulin_left(SigmaBase[0],PPiv);
-				SigmaBase[0].write(std::cout,_field);
+				SigmaBase[0].write(std::cout,field());
 				_BMD.mulin_left(SigmaBase[0], PTr);
-				SigmaBase[0].write(std::cout,_field);
+				SigmaBase[0].write(std::cout,field());
 				_BMD.mulin_left(SigmaBase[0], PTrT);
 				_BMD.mulin_left(SigmaBase[0],PPivT);
 				TransposedBlasMatrix<BlasPermutation<size_t> > BPerm1T(BPerm1);
@@ -1573,7 +1578,7 @@ namespace LinBox
 
 				// Update SigmaBase
 				for (size_t i=0;i<maxs+1;++i) {
-					//SigmaBase[0].write(std::cout,_field);
+					//SigmaBase[0].write(std::cout,field());
 					// permute SigmaBase
 					_BMD.mulin_right(BPerm1, SigmaBase[i]);
 
@@ -1598,14 +1603,14 @@ namespace LinBox
 					BlasMatrix<Field> S_bottom_left (SigmaBase[i], rank,0,m-rank,lsize);
 #if 0
 					if (i==0){
-						S_bottom_left.write(std::cout,_field);
+						S_bottom_left.write(std::cout,field());
 					}
 #endif
 					// deal with the left part of S_bottom
 					_BMD.axmyin(S_bottom_left, G, S_top_left);
 #if 0
 					if (i==0){
-						S_bottom_left.write(std::cout,_field);
+						S_bottom_left.write(std::cout,field());
 					}
 #endif
 
@@ -1625,7 +1630,7 @@ namespace LinBox
 						_BMD.mulin_right(Q, SigmaBase[i]);
 #if 0
 					if (i==0){
-						S_bottom_left.write(std::cout,_field);
+						S_bottom_left.write(std::cout,field());
 					}
 #endif
 				}
@@ -1780,10 +1785,10 @@ namespace LinBox
 				for (size_t i=0;i<rank;++i){
 					for (int j= (int) size-2;j>=0; --j){
 						for (size_t l=0;l<m;++l)
-							_field.assign(SigmaBase[j+1].refEntry(*(Qt.getPointer()+i),l), SigmaBase[j].getEntry(*(Qt.getPointer()+i),l));
+							field().assign(SigmaBase[(size_t)j+1].refEntry(*(Qt.getPointer()+i),l), SigmaBase[(size_t)j].getEntry(*(Qt.getPointer()+i),l));
 					}
 					for (size_t l=0;l<m;++l)
-						_field.assign(SigmaBase[0].refEntry(*(Qt.getPointer()+i),l),_field.zero);
+						field().assign(SigmaBase[0].refEntry(*(Qt.getPointer()+i),l),field().zero);
 				}
 #ifdef _BM_TIMING
 				chrono.stop();
@@ -1796,7 +1801,7 @@ namespace LinBox
 				for (size_t i=0;i<rank;++i){
 				for (int j= (int) length-2;j>= (int) k; j--){
 				for (size_t l=0;l<n;++l)
-				_field.assign(Residual[j+1].refEntry(*(Qt.getPointer()+i),l), Residual[j].getEntry(*(Qt.getPointer()+i),l));
+				field().assign(Residual[(size_t)j+1].refEntry(*(Qt.getPointer()+i),l), Residual[(size_t)j].getEntry(*(Qt.getPointer()+i),l));
 				}
 				}
 				*/
@@ -1835,14 +1840,14 @@ namespace LinBox
 			size_t m,n;
 			m = PowerSerie[0].rowdim();
 			n = PowerSerie[0].coldim();
-			const Coefficient ZeroSigma(m,m);
-			const Coefficient ZeroSerie(m,n);
+			const Coefficient ZeroSigma(field(),m,m);
+			const Coefficient ZeroSerie(field(),m,n);
 			PowerSerie.resize(degree, ZeroSerie);
 
 			if (degree == 0) {
-				Coefficient Identity(m,m);
+				Coefficient Identity(field(),m,m);
 				for (size_t i=0;i< m;++i)
-					Identity.setEntry(i,i,_field.one);
+					Identity.setEntry(i,i,field().one);
 				SigmaBase[0]=Identity;
 			}
 			else {
@@ -1887,7 +1892,8 @@ namespace LinBox
 					tUpdateSerie.clear();
 					tUpdateSerie.start();
 #endif
-					/*
+
+#if 0
 					//write_maple("Sigma1", Sigma1);
 					Sigma1.resize(degree1+1, ZeroSigma);
 
@@ -1900,10 +1906,10 @@ namespace LinBox
 					Serie[i] = PowerSerie[i];
 
 					PM_domain.midproduct(Serie2, Sigma1, Serie);
-					//ClassicMulDomain<Field, std::vector<Coefficient> > CM_domain(_field);
+					//ClassicMulDomain<Field, std::vector<Coefficient> > CM_domain(field());
 					//CM_domain.midproduct(Serie2, Sigma1, Serie);
 					Serie2.resize(degree2, ZeroSerie);
-					*/
+#endif
 					std::vector<Coefficient> Serie2(degree2,ZeroSerie);
 					UpdateSerie(Serie2, Sigma1, PowerSerie, degree1, degree2);
 
@@ -1979,17 +1985,17 @@ namespace LinBox
 			size_t m,n;
 			m = OldSerie[0].rowdim();
 			n = OldSerie[0].coldim();
-			const Coefficient ZeroSigma(m,m);
-			const Coefficient ZeroSerie(m,n);
+			const Coefficient ZeroSigma(field(),m,m);
+			const Coefficient ZeroSerie(field(),m,n);
 			size_t Ssize = SigmaBase.size();
 
 			if (SigmaBase.size() < 5){
 
 				// do the calculation by hand
 				for (size_t j=degree1;j<degree1+degree2;++j){
-					_BMD.mul(NewSerie[j-degree1], SigmaBase[0], OldSerie[j]);
+					_BMD.mul(NewSerie[(size_t)j-degree1], SigmaBase[0], OldSerie[(size_t)j]);
 					for (size_t i=1;i<Ssize; ++i)
-						_BMD.axpyin(NewSerie[j-degree1], SigmaBase[i], OldSerie[j-i]);
+						_BMD.axpyin(NewSerie[(size_t)j-degree1], SigmaBase[i], OldSerie[(size_t)j-i]);
 				}
 			}
 			else{
@@ -2023,13 +2029,13 @@ namespace LinBox
 			for (size_t i=0;i<m-1;++i){
 				std::cout<<"[";
 				for (size_t j=0;j<n-1;++j)
-					_field.write(std::cout,C.getEntry(i,j))<<",";
-				_field.write(std::cout,C.getEntry(i,n-1))<<"] , ";
+					field().write(std::cout,C.getEntry(i,j))<<",";
+				field().write(std::cout,C.getEntry(i,n-1))<<"] , ";
 			}
 			std::cout<<"[";
 			for (size_t j=0;j<n-1;++j)
-				_field.write(std::cout,C.getEntry(m-1,j))<<",";
-			_field.write(std::cout, C.getEntry(m-1,n-1))<<"]])]); ";
+				field().write(std::cout,C.getEntry(m-1,j))<<",";
+			field().write(std::cout, C.getEntry(m-1,n-1))<<"]])]); ";
 
 		}
 
@@ -2046,26 +2052,26 @@ namespace LinBox
 				for (size_t i=0;i<m-1;++i){
 					std::cout<<"[";
 					for (size_t j=0;j<n-1;++j)
-						_field.write(std::cout,P[k].getEntry(i,j))<<",";
-					_field.write(std::cout, P[k].getEntry(i,n-1))<<"] , ";
+						field().write(std::cout,P[k].getEntry(i,j))<<",";
+					field().write(std::cout, P[k].getEntry(i,n-1))<<"] , ";
 				}
 				std::cout<<"[";
 				for (size_t j=0;j<n-1;++j)
-					_field.write(std::cout,P[k].getEntry(m-1,j))<<",";
-				_field.write(std::cout, P[k].getEntry(m-1,n-1))<<"]]) , ";
+					field().write(std::cout,P[k].getEntry(m-1,j))<<",";
+				field().write(std::cout, P[k].getEntry(m-1,n-1))<<"]]) , ";
 			}
 
 			std::cout<<"Matrix([";
 			for (size_t i=0;i<m-1;++i){
 				std::cout<<"[";
 				for (size_t j=0;j<n-1;++j)
-					_field.write(std::cout,P[P.size()-1].getEntry(i,j))<<",";
-				_field.write(std::cout, P[P.size()-1].getEntry(i,n-1))<<"] , ";
+					field().write(std::cout,P[P.size()-1].getEntry(i,j))<<",";
+				field().write(std::cout, P[P.size()-1].getEntry(i,n-1))<<"] , ";
 			}
 			std::cout<<"[";
 			for (size_t j=0;j<n-1;++j)
-				_field.write(std::cout,P[P.size()-1].getEntry(m-1,j))<<",";
-			_field.write(std::cout, P[P.size()-1].getEntry(m-1,n-1))<<"]])]); \n\n";
+				field().write(std::cout,P[P.size()-1].getEntry(m-1,j))<<",";
+			field().write(std::cout, P[P.size()-1].getEntry(m-1,n-1))<<"]])]); \n\n";
 		}
 
 
@@ -2078,11 +2084,10 @@ namespace LinBox
 #endif //__LINBOX_sigma_basis_H
 
 
-// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,:0,t0,+0,=s
 // Local Variables:
 // mode: C++
 // tab-width: 8
 // indent-tabs-mode: nil
 // c-basic-offset: 8
 // End:
-
+// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,\:0,t0,+0,=s
