@@ -26,12 +26,12 @@
 #define __LINBOX_signature_H
 /* Function related to the signature computation of symmetric matrices */
 
-#include "linbox/field/modular.h"
+#include "linbox/ring/modular.h"
 #include "linbox/algorithms/cra-early-multip.h"
 #include <fflas-ffpack/ffpack/ffpack.h>
 #include "linbox/randiter/random-prime.h"
-#include "linbox/matrix/blas-matrix.h"
-#include "linbox/algorithms/blas-domain.h"
+#include "linbox/matrix/dense-matrix.h"
+#include "linbox/matrix/matrix-domain.h"
 #include "linbox/solutions/minpoly.h"
 
 namespace LinBox
@@ -49,20 +49,21 @@ namespace LinBox
 		template <class Matrix>
 		static bool isPosDef (const Matrix& M, const BLAS_LPM_Method& meth)
 		{
-			RandomPrimeIterator::setSeed(time(0));
+			RandomPrimeIterator::setSeed((size_t)time(0));
 			size_t n = M. rowdim();
 			std::vector<int> P;
 			symmetricLU (P, M);
 			if (P. size () < n)
 				return false;
 
-			typedef typename Matrix::Field::Element Int;
-			std::vector<Int> D(n);
+			// typedef typename Matrix::Field::Element Int;
+			// std::vector<Int> D(n);
+			BlasVector<typename Matrix::Field> D(M.field(),n);
 			semiD(D, M);
 
 			//std::cout << "All principal minors are: [";
 			//for (int i = 0; i < n; ++ i)
-			//	std::cout << D[i] << ", ";
+			//	std::cout << D[(size_t)i] << ", ";
 			//std::cout << "]\n";
 
 			if (allPos(D)) return true;
@@ -95,10 +96,10 @@ namespace LinBox
 		template <class Matrix>
 		static bool isPosSemiDef (const Matrix& M, const BLAS_LPM_Method& meth)
 		{
-			RandomPrimeIterator::setSeed(time(0));
+			RandomPrimeIterator::setSeed((size_t)time(0));
 			size_t n = M. rowdim();
 			std::vector<int> P;
-			size_t r = rank_random (M);
+			size_t r = (size_t)rank_random (M);
 			//std::clog << "Rank:= " << r << std::endl;
 			if (r == 0)
 				return true;
@@ -118,10 +119,10 @@ namespace LinBox
 				Matrix PM (R, P.size(), P.size());
 				typename Matrix::RowIterator cur_r; int j = 0;
 				for (cur_r = PM. rowBegin(); cur_r != PM. rowEnd(); ++ cur_r, ++j) {
-					typename Matrix::ConstRowIterator m_r = M. rowBegin() + P[j];
+					typename Matrix::ConstRowIterator m_r = M. rowBegin() + P[(size_t)j];
 					for (size_t k = 0; k < P.size(); ++ k)
 						R. assign (cur_r -> operator[] (k),
-							   m_r -> operator[] (P[k]));
+							   m_r -> operator[] ((size_t)P[(size_t)k]));
 				}
 				semiD (D, PM);
 			}
@@ -192,26 +193,28 @@ namespace LinBox
 			//M. write (std::cout);
 			typedef typename Matrix::Field Ring;
 			typedef typename Ring::Element Integer_t;
-			typedef Modular<double> Field;
+			typedef Givaro::Modular<double> Field;
 			typedef Field::Element Element;
 
 			size_t n = M. rowdim();
+	                RandomPrimeIterator primeg;
+                	if( ! primeg.template setBitsDelayedField<Field>(n) )
+                       		primeg.template setBitsField<Field>();
 
-			integer mmodulus;
-			FieldTraits<Field>::maxModulus(mmodulus);
-			long bit1 = (long) floor (log((double)mmodulus)/M_LN2);
-			long bit2 = (long) floor (log(sqrt(double(4503599627370496LL/n)))/M_LN2);
-			RandomPrimeIterator primeg(unsigned(bit1 < bit2 ? bit1 : bit2));
 
-			Field::Element* FA = new Field::Element[n*n];
+
+			Element* FA = new Element[n*n];
 			size_t* P= new size_t[n], *PQ = new size_t[n];
 			size_t* P_p, * PQ_p;
 
-			Field::Element* p; Field::Element tmp;
+			Element* p; Element tmp;
 			EarlyMultipCRA< Field > cra(3UL);
 
 			Integer_t m = 1;
-			std::vector<Field::Element> v(n);
+			// std::vector<Element> v(n);
+			typedef Givaro::ZRing<Element> NoField;
+			NoField unF ;
+			BlasVector<NoField> v(unF,n);
 			size_t j = 0;
 			Field K2;
 			bool faithful = true;
@@ -229,7 +232,7 @@ namespace LinBox
 					K1. init (*p, *raw_p);
 
 				//clog << "\rComputing lup mod " << prime << ". ";
-				FFPACK::LUdivine((typename Field::Father_t)K1, FFLAS::FflasNonUnit, FFLAS::FflasNoTrans, n, n, FA, n, P, PQ, FFPACK::FfpackLQUP);
+				FFPACK::LUdivine(K1, FFLAS::FflasNonUnit, FFLAS::FflasNoTrans, n, n, FA, n, P, PQ);//, FFPACK::FfpackLQUP);
 
 				faithful = true;
 				for ( j = 0, P_p = P, PQ_p = PQ; j < n; ++ j, ++ P_p, ++ PQ_p)
@@ -239,14 +242,16 @@ namespace LinBox
 					}
 
 			} while(! faithful);
-			K2. init (tmp, 1UL);
+			K2. assign(tmp, K2.one);
 
-			typename std::vector<Field::Element>::iterator vp;
+			// typename std::vector<Element>::iterator vp;
+			typename BlasVector<NoField>::iterator vp;
 			for (j = 0, vp = v.begin(); vp != v.end(); ++j, ++vp) {
 				K2.mulin(tmp, *(FA + (j * n + j)));
 				K2.assign(*vp, tmp);
 			}
-			cra. initialize(K2, v);
+			// BlasVector<Field> v2(K2,v);//!@bug should not do it like that...
+			cra. initialize(K2, v );
 
 			while (! cra.terminated() ){
 				// get a prime.
@@ -257,7 +262,7 @@ namespace LinBox
 					K3. init (*p, *raw_p);
 
 				//clog << "\rComputing lup mod " << prime << ". ";
-				FFPACK::LUdivine((typename Field::Father_t)K3, FFLAS::FflasNonUnit, FFLAS::FflasNoTrans, n, n, FA, n, P, PQ, FFPACK::FfpackLQUP);
+				FFPACK::LUdivine(K3, FFLAS::FflasNonUnit, FFLAS::FflasNoTrans, n, n, FA, n, P, PQ);//, FFPACK::FfpackLQUP);
 
 				faithful = true;
 				for ( j = 0, P_p = P, PQ_p = PQ; j < n; ++ j, ++ P_p, ++ PQ_p)
@@ -271,7 +276,7 @@ namespace LinBox
 					continue;
 				}
 
-				K3. init (tmp, 1UL);
+				K3. assign(tmp, K3.one);
 
 				for (j = 0, vp = v.begin(); vp != v.end(); ++j, ++vp) {
 					K3.mulin(tmp, *(FA + (j * n + j)));
@@ -283,6 +288,7 @@ namespace LinBox
 					std::cout << v[l] << ", ";
 				std::cout << "]\n";
 #endif
+				// BlasVector<Field> v3(K3,v); //!@bug should not be doing that...
 				cra. progress(K3, v);
 			}
 
@@ -301,29 +307,28 @@ namespace LinBox
 		static Vector& symmetricLU (Vector& v, const Matrix& IM)
 		{
 
-			typedef Modular<int32_t> Field;
-			// typedef Modular<double> Field;
+			typedef Givaro::Modular<int32_t> Field;
+			// typedef Givaro::Modular<double> Field;
 			typedef Field::Element Element;
 			typedef BlasMatrix<Field> FMatrix;
-			RandomPrimeIterator primeg(20);
-			Field F ((unsigned long)*primeg);
+			RandomPrimeIterator primeg; primeg.template setBitsField<Field>();
+			Field F (*primeg);
 			FMatrix FM(F, IM.rowdim(), IM.coldim());
 			//std::cout << "Random prime " << p << "\n";
 
-			Element zero; F. init (zero, 0);
-			MatrixHom::map (FM, IM, F);
+			MatrixHom::map (FM, IM);
 			VectorDomain<Field> VD(F);
 			FMatrix& M = FM;
 
 			//typename FMatrix::RowIterator cur_r, tmp_r;
-			typedef FMatrix::Row Row;
+			// typedef FMatrix::Row Row;
 			//the index is 0-based.
 			int i = 0;
 			int n = (int) M. rowdim();
-			std::vector<int> P(n);
+			std::vector<int> P((size_t)n);
 
 			for (i = 0; i < n; ++ i)
-				P[i] = i;
+				P[(size_t)i] = i;
 
 			//M. write(std::cout);
 			for (i = 0; i < n; ++ i) {
@@ -331,7 +336,7 @@ namespace LinBox
 				int j;
 				//find a pivot
 				for (j = i; j < n; ++ j) {
-					if (!F. isZero(M[j][j])) break;
+					if (!F. isZero(M[(size_t)j][(size_t)j])) break;
 				}
 
 				//no piviot
@@ -345,15 +350,15 @@ namespace LinBox
 				//std::cout << "Pivot= " << j << '\n';
 				//M. write(std::cout);
 
-				P[i] = j;
+				P[(size_t)i] = j;
 				Element tmp;
-				F. inv (tmp, M[i][i]);
+				F. inv (tmp, M[(size_t)i][(size_t)i]);
 				F. negin(tmp);
 				VD. mulin(*(M. rowBegin() + i), tmp);
 				//M. write(std::cout);
 
 				for (j = i + 1; j < n; ++ j) {
-					F. assign (tmp,  M[j][i]);
+					F. assign (tmp,  M[(size_t)j][(size_t)i]);
 					VD. axpyin (*(M. rowBegin() + j), tmp,
 						    *(M. rowBegin() + i));
 				}
@@ -361,20 +366,20 @@ namespace LinBox
 				//not necessary
 				//M. write(std::cout);
 				for (j = i + 1; j < n; ++ j)
-					F. assign (M[i][j], zero);
+					F. assign (M[(size_t)i][(size_t)j], F.zero);
 			}
 
-			v. resize (n);
+			v. resize ((size_t)n);
 			std::vector<int>::iterator i_p; int j;
 			for (i_p = v. begin(), j = 0; i_p != v. end(); ++ i_p, ++ j)
 				*i_p = j;
 
 			for (j = 0; j < i; ++ j) {
-				if (j != P[j])
-					std::swap (v[j], v[P[j]]);
+				if (j != P[(size_t)j])
+					std::swap (v[(size_t)j], v[(size_t)P[(size_t)j]]);
 			}
 
-			v. resize (i);
+			v. resize ((size_t)i);
 
 			//std::cout << "Pseud-rank: " << i << "\n[";
 			//for (i_p = v. begin(); i_p != v. end(); ++ i_p)
@@ -390,30 +395,28 @@ namespace LinBox
 		static long rank_random (const Matrix& M)
 		{
 
-			typedef typename Matrix::Field Ring;
+			// typedef typename Matrix::Field Ring;
 			// typedef typename Ring::Element Integer_t;
-			typedef Modular<double> Field;
-			typedef Field::Element Element;
+			typedef Givaro::Modular<double> Field;
+			// typedef Field::Element Element;
 
 			int n = (int)M. rowdim();
 
-			integer mmodulus;
-			FieldTraits<Field>::maxModulus(mmodulus);
-			long bit1 = (long) floor (log((double)mmodulus)/M_LN2);
-			long bit2 = (long) floor (log(sqrt(double(4503599627370496LL/n)))/M_LN2);
-			RandomPrimeIterator primeg((unsigned)(bit1 < bit2 ? bit1 : bit2));
-
 			Field::Element* FA = new Field::Element[n*n], *p;
 
 			// get a prime.
 			// Compute the rank mod that prime. Accumulate into v with CRA.
+                        RandomPrimeIterator primeg;
+                        if( ! primeg.template setBitsDelayedField<Field>(n) )
+                                primeg.template setBitsField<Field>();
+
 			Field K(*primeg);
 
 			typename Matrix::ConstIterator raw_p;
 			for (p = FA, raw_p = M. Begin(); p != FA + (n*n); ++ p, ++ raw_p)
 				K. init (*p, *raw_p);
 
-			long r = FFPACK::Rank((typename Field::Father_t) K, n, n, FA, n);
+			long r = (long)FFPACK::Rank( K, (size_t)n, (size_t)n, FA, (size_t)n);
 
 			delete[] FA;
 			return r;
