@@ -35,20 +35,20 @@
  * which one to use in different circumstances
  * ------------------------------------
  *
- * 
+ *
  * ========LICENCE========
  * This file is part of the library LinBox.
- * 
+ *
  * LinBox is free software: you can redistribute it and/or modify
  * it under the terms of the  GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
- * 
+ *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
@@ -69,7 +69,7 @@
 
 #include "linbox/blackbox/archetype.h"
 #include "linbox/blackbox/squarize.h"
-#include "linbox/blackbox/sparse.h"
+#include "linbox/matrix/sparse-matrix.h"
 #include "linbox/util/debug.h"
 #include "linbox/vector/vector-domain.h"
 #include "linbox/solutions/methods.h"
@@ -130,23 +130,22 @@ namespace LinBox
 #endif // INCLUDE_TIMING
 		}
 
-
-
 		commentator().stop ("done", NULL, "minpoly");
 
 		return P;
 	}
 }
 
-#ifdef __LINBOX_HAVE_GIVARO
 #ifndef LINBOX_EXTENSION_DEGREE_MAX
 #define LINBOX_EXTENSION_DEGREE_MAX 19
 #endif
 
-#include "linbox/blackbox/sparse.h"
-#include "linbox/field/modular.h"
+#include <givaro/extension.h>
+#include <givaro/gfq.h>
+
+#include "linbox/matrix/sparse-matrix.h"
+#include "linbox/ring/modular.h"
 #include "linbox/algorithms/matrix-hom.h"
-#include "linbox/field/givaro.h"
 #include "linbox/field/map.h"
 
 namespace LinBox
@@ -163,34 +162,34 @@ namespace LinBox
 		const Field& F = A.field();
 		integer a,c; F.cardinality(a); F.characteristic(c);
 		if (a != c) {
-			unsigned long extend = (unsigned long)FF_EXPONENT_MAX(a,(integer)LINBOX_EXTENSION_DEGREE_MAX);
+			uint64_t extend = (uint64_t)FF_EXPONENT_MAX(a,(integer)LINBOX_EXTENSION_DEGREE_MAX);
 			if (extend > 1) {
 				commentator().report (Commentator::LEVEL_ALWAYS,INTERNAL_WARNING) << "Extension of degree " << extend << std::endl;
-				GivaroExtension<Field> EF( F, extend);
+				Givaro::Extension<Field> EF( F, extend);
 
-				typedef typename Blackbox::template rebind< GivaroExtension<Field>  >::other FBlackbox;
+				typedef typename Blackbox::template rebind< Givaro::Extension<Field>  >::other FBlackbox;
 
 				FBlackbox Ap(A, EF);
 
-				std::vector< typename GivaroExtension<Field>::Element > eP;
+				BlasVector< Givaro::Extension<Field> > eP(EF);
 				minpoly(eP, Ap, tag, Method::Wiedemann(M));
 
-				return PreMap<Field, GivaroExtension<Field> >(F,EF)(P, eP);
+				return PreMap<Field, Givaro::Extension<Field> >(F,EF)(P, eP);
 			}
 			else
 				return minpoly(P, A, tag, Method::Wiedemann(M));
-
 		}
 		else {
-			unsigned long extend = (unsigned long)FF_EXPONENT_MAX(c,(integer)LINBOX_EXTENSION_DEGREE_MAX);
+			uint64_t extend = (uint64_t)FF_EXPONENT_MAX(c,(integer)LINBOX_EXTENSION_DEGREE_MAX);
 			if (extend > 1) {
 				commentator().report (Commentator::LEVEL_ALWAYS,INTERNAL_WARNING) << "Word size extension : " << extend << std::endl;
-				GivaroGfq EF( (unsigned long)c, extend);
-				typedef typename Blackbox::template rebind< GivaroGfq >::other FBlackbox;
+				typedef Givaro::GFqDom<int64_t> Fld;
+				Fld EF( (uint64_t)c, extend);
+				typedef typename Blackbox::template rebind< Fld >::other FBlackbox;
 				FBlackbox Ap(A, EF);
-				std::vector< typename GivaroGfq::Element > eP;
+				BlasVector< Fld > eP(EF);
 				minpoly(eP, Ap, tag, Method::Wiedemann(M));
-				return PreMap<Field, GivaroGfq >(F,EF)(P, eP);
+				return PreMap<Field, Fld >(F,EF)(P, eP);
 
 			}
 			else
@@ -198,8 +197,8 @@ namespace LinBox
 		}
 	}
 }
-#else
-namespace LinBox
+
+/*namespace LinBox
 {
 	// The minpoly with BlackBox Method
 	template<class Polynomial, class Blackbox>
@@ -212,8 +211,7 @@ namespace LinBox
 		commentator().report (Commentator::LEVEL_ALWAYS,INTERNAL_WARNING) << " WARNING, no extension available, returning only a factor of the minpoly\n";
 		return minpoly(P, A, tag, Method::Wiedemann (M));
 	}
-}
-#endif
+}*/
 
 namespace LinBox
 {
@@ -240,7 +238,7 @@ namespace LinBox
 		 *               options for the solver
 		 */
 		WiedemannSolver (const Field &F, const WiedemannTraits &traits) :
-			_traits (traits), _field (F), _randiter (F), _VD (F)
+			_traits (traits), _field (&F), _randiter (F), _VD (F)
 		{}
 
 		/*! Constructor with a random iterator.
@@ -253,7 +251,7 @@ namespace LinBox
 		WiedemannSolver (const Field &F,
 				 const WiedemannTraits &traits,
 				 typename Field::RandIter r) :
-			_traits (traits), _field (F), _randiter (r), _VD (F)
+			_traits (traits), _field (&F), _randiter (r), _VD (F)
 		{}
 
 		/// \ingroup algorithms
@@ -354,6 +352,7 @@ namespace LinBox
 
 		//@}
 
+		inline const Field & field() const { return *_field; }
 
 	private:
 
@@ -361,7 +360,7 @@ namespace LinBox
 		SparseMatrix<Field> *makeLambdaSparseMatrix (size_t m);
 
 		WiedemannTraits                      _traits;
-		const Field                         &_field;
+		const Field                         *_field;
 		typename Field::RandIter             _randiter;
 		VectorDomain<Field>                  _VD;
 	};
@@ -373,11 +372,10 @@ namespace LinBox
 #endif //  __LINBOX_wiedemann_H
 
 
-// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,:0,t0,+0,=s
 // Local Variables:
 // mode: C++
 // tab-width: 8
 // indent-tabs-mode: nil
 // c-basic-offset: 8
 // End:
-
+// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,\:0,t0,+0,=s
