@@ -31,7 +31,7 @@
 #include "linbox/util/debug.h"
 #include "linbox/linbox-config.h"
 #include "linbox/blackbox/blackbox-interface.h"
-#include "linbox/matrix/blas-matrix.h"
+#include "linbox/vector/blas-vector.h"
 
 namespace LinBox
 {
@@ -81,7 +81,7 @@ namespace LinBox
 		 * @param B blackbox
 		 */
 		Compose (const Blackbox1 &A, const Blackbox2 &B) :
-			_A_ptr(&A), _B_ptr(&B)
+			_A_ptr(&A), _B_ptr(&B),_z(A.field())
 		{
 			// Rich Seagraves - "It seems VectorWrapper somehow
 			// became depricated.  Makes the assumption that
@@ -97,7 +97,7 @@ namespace LinBox
 		 * @param B_ptr blackbox
 		 */
 		Compose (const Blackbox1 *A_ptr, const Blackbox2 *B_ptr) :
-			_A_ptr(A_ptr), _B_ptr(B_ptr)
+			_A_ptr(A_ptr), _B_ptr(B_ptr),_z(A_ptr->field())
 		{
 			linbox_check (A_ptr != (Blackbox1 *) 0);
 			linbox_check (B_ptr != (Blackbox2 *) 0);
@@ -113,8 +113,7 @@ namespace LinBox
 		 * @param[in] Mat blackbox to copy.
 		 */
 		Compose (const Compose<Blackbox1, Blackbox2>& Mat) :
-			_A_ptr ( Mat._A_ptr), _B_ptr ( Mat._B_ptr)
-			//{ VectorWrapper::ensureDim (_z, _A_ptr->coldim ()); }
+			_A_ptr ( Mat._A_ptr), _B_ptr ( Mat._B_ptr),_z(Mat.field())
 		{
 			_z.resize(_A_ptr->coldim());
 		}
@@ -237,7 +236,7 @@ namespace LinBox
 		const Blackbox2 *_B_ptr;
 
 		// local intermediate vector
-		mutable std::vector<Element> _z;
+		mutable BlasVector<Field> _z;
 	};
 
 	/// specialization for _Blackbox1 = _Blackbox2
@@ -298,14 +297,18 @@ namespace LinBox
 			typename std::vector<std::vector<Element> >::reverse_iterator z_p, pz_p;
 			b_p = _BlackboxL.rbegin();
 			pz_p = z_p = _zl.rbegin();
+			typedef BlasSubvector<BlasVector<Field, typename Vector<Field>::Dense> > BSub;
+			BSub pz_p_vec(field(),*pz_p);
 
-			(*b_p) -> apply(*pz_p, x);
+			(*b_p) -> apply(pz_p_vec, x);
 			++ b_p;  ++ z_p;
 
-			for (; z_p != _zl.rend(); ++ b_p, ++ z_p, ++ pz_p)
-				(*b_p) -> apply (*z_p,*pz_p);
+			for (; z_p != _zl.rend(); ++ b_p, ++ z_p, ++ pz_p) {
+				 BSub z_p_vec(field(),*z_p);
+				(*b_p) -> apply (z_p_vec,pz_p_vec);
+			}
 
-			(*b_p) -> apply(y, *pz_p);
+			(*b_p) -> apply(y, pz_p_vec);
 
 			return y;
 		}
@@ -411,16 +414,20 @@ namespace LinBox
 		typedef Compose<IMatrix, IMatrix> value_type;
 	};
 
+
+	template<class Field, class Rep>
+	class BlasMatrix ;
+
 	/// used in smith-binary, for example
-	template<class Field>
-	class ComposeTraits<  BlasMatrix<Field> > {
+	template<class Field, class Rep>
+	class ComposeTraits<  BlasMatrix<Field, Rep> > {
 	public:
 
 		// define the return value type
-		typedef BlasMatrix<Field> value_type;
+		typedef BlasMatrix<Field, Rep> value_type;
 	};
 
-}
+} // LinBox
 
 
 namespace LinBox
@@ -460,6 +467,7 @@ namespace LinBox
 		 */
 		ComposeOwner (const Blackbox1 &A, const Blackbox2 &B) :
 			_A_data(A), _B_data(B)
+			,_z(A.field())
 		{
 			// Rich Seagraves - "It seems VectorWrapper somehow
 			// became depricated.  Makes the assumption that
@@ -474,6 +482,7 @@ namespace LinBox
 		 */
 		ComposeOwner (const Blackbox1 *A_data, const Blackbox2 *B_data) :
 			_A_data(*A_data), _B_data(*B_data)
+			,_z(A_data->field())
 		{
 			linbox_check (A_data != (Blackbox1 *) 0);
 			linbox_check (B_data != (Blackbox2 *) 0);
@@ -490,6 +499,7 @@ namespace LinBox
 		 */
 		ComposeOwner (const ComposeOwner<Blackbox1, Blackbox2>& Mat) :
 			_A_data ( Mat.getLeftData()), _B_data ( Mat.getRightData())
+			,_z(Mat.field())
 		{
 			_z.resize(_A_data.coldim());
 		}
@@ -555,7 +565,7 @@ namespace LinBox
 		ComposeOwner (const Compose<_BBt1, _BBt2> &Mat, const Field& F) :
 			_A_data(*(Mat.getLeftPtr()), F),
 			_B_data(*(Mat.getRightPtr()), F),
-			_z(_A_data.coldim())
+			_z(F,_A_data.coldim())
 		{
 			typename Compose<_BBt1, _BBt2>::template rebind<Field>()(*this,Mat);
 		}
@@ -564,7 +574,7 @@ namespace LinBox
 		ComposeOwner (const ComposeOwner<_BBt1, _BBt2> &Mat, const Field& F) :
 			_A_data(Mat.getLeftData(), F),
 			_B_data(Mat.getRightData(), F) ,
-			_z(_A_data.coldim())
+			_z(F,_A_data.coldim())
 		{
 			typename ComposeOwner<_BBt1, _BBt2>::template rebind<Field>()(*this,Mat);
 		}
@@ -611,20 +621,19 @@ namespace LinBox
 		Blackbox2 _B_data;
 
 		// local intermediate vector
-		mutable std::vector<Element> _z;
+		mutable BlasVector<Field> _z;
 	};
 
-}
+} // LinBox
 
 
 #endif // __LINBOX_compose_H
 
 
-// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,:0,t0,+0,=s
 // Local Variables:
 // mode: C++
 // tab-width: 8
 // indent-tabs-mode: nil
 // c-basic-offset: 8
 // End:
-
+// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,\:0,t0,+0,=s
