@@ -44,7 +44,7 @@
 #include "linbox/util/debug.h"
 #include "linbox/util/error.h"
 #include "linbox/vector/vector-domain.h"
-#include "linbox/matrix/blas-matrix.h"
+#include "linbox/matrix/dense-matrix.h"
 #include "linbox/blackbox/blackbox-interface.h"
 
 
@@ -140,12 +140,12 @@ namespace LinBox
 		template<class OutVector, class InVector>
 		OutVector& apply (OutVector &y, const InVector& x) const
 		{
-			std::fill (_z.begin (), _z.begin () + _col, _BB->field().zero);
-			std::fill (_z.begin () + _col + _coldim, _z.end (), _BB->field().zero);
+			std::fill (_z.begin (), _z.begin () + (ptrdiff_t)_col, _BB->field().zero);
+			std::fill (_z.begin () + (ptrdiff_t)(_col + _coldim), _z.end (), _BB->field().zero);
 
-			copy (x.begin (), x.end (), _z.begin () + _col);  // Copying. Yuck.
+			copy (x.begin (), x.end (), _z.begin () + (ptrdiff_t)_col);  // Copying. Yuck.
 			_BB->apply (_y, _z);
-			copy (_y.begin () + _row, _y.begin () + _row + _rowdim, y.begin ());
+			copy (_y.begin () + (ptrdiff_t)_row, _y.begin () + (ptrdiff_t)(_row + _rowdim), y.begin ());
 			return y;
 		}
 
@@ -161,12 +161,12 @@ namespace LinBox
 		template<class OutVector, class InVector>
 		OutVector& applyTranspose (OutVector &y, const InVector& x) const
 		{
-			std::fill (_y.begin (), _y.begin () + _row, _BB->field().zero);
-			std::fill (_y.begin () + _row + _rowdim, _y.end (), _BB->field().zero);
+			std::fill (_y.begin (), _y.begin () + (ptrdiff_t)_row, _BB->field().zero);
+			std::fill (_y.begin () + (ptrdiff_t)(_row + _rowdim), _y.end (), _BB->field().zero);
 
-			copy (x.begin (), x.end (), _y.begin () + _row);  // Copying. Yuck.
+			copy (x.begin (), x.end (), _y.begin () + (ptrdiff_t)_row);  // Copying. Yuck.
 			_BB->applyTranspose (_z, _y);
-			copy (_z.begin () + _col, _z.begin () + _col + _coldim, y.begin ());
+			copy (_z.begin () + (ptrdiff_t)_col, _z.begin () + (ptrdiff_t)(_col + _coldim), y.begin ());
 			return y;
 		}
 
@@ -265,12 +265,13 @@ namespace LinBox
 
 	/** Specialization for BlasMatrix */
 	template<class _Field>
-	class Submatrix<BlasMatrix<_Field>, VectorCategories::DenseVectorTag> : public BlasSubmatrix<_Field> {
+	class Submatrix<BlasMatrix<_Field>, VectorCategories::DenseVectorTag> : public BlasSubmatrix<BlasMatrix<_Field> > {
 	public:
 
 		typedef _Field Field;
-		typedef Submatrix<BlasMatrix<_Field>, VectorCategories::DenseVectorTag> Self_t;
-		typedef BlasSubmatrix<_Field> Father_t;
+		typedef BlasMatrix<_Field> Matrix ;
+		typedef Submatrix<Matrix, VectorCategories::DenseVectorTag> Self_t;
+		typedef BlasSubmatrix<Matrix> Father_t;
 
 	private:
 
@@ -294,7 +295,7 @@ namespace LinBox
 			   size_t col,
 			   size_t Rowdim,
 			   size_t Coldim) :
-			BlasSubmatrix<Field>(const_cast<BlasMatrix<Field>& >(*Mat), row, col, Rowdim, Coldim),
+			BlasSubmatrix<Matrix>(const_cast<BlasMatrix<Field>& >(*Mat), row, col, Rowdim, Coldim),
 			f(Mat -> field()), vd(Mat -> field())
 		{ }
 
@@ -310,7 +311,7 @@ namespace LinBox
 			   size_t col,
 			   size_t Rowdim,
 			   size_t Coldim) :
-			BlasSubmatrix<Field>(const_cast<BlasMatrix<Field>& >(Mat), row, col, Rowdim, Coldim),
+			BlasSubmatrix<Matrix>(const_cast<BlasMatrix<Field>& >(Mat), row, col, Rowdim, Coldim),
 			f(Mat.field()), vd(Mat.field())
 		{ }
 
@@ -327,10 +328,10 @@ namespace LinBox
 			   size_t col,
 			   size_t Rowdim,
 			   size_t Coldim ) :
-			BlasSubmatrix<Field> (const_cast<Submatrix<BlasMatrix<Field> >&>(*SM), row, col, Rowdim, Coldim),
+			BlasSubmatrix<Matrix> (const_cast<Submatrix<BlasMatrix<Field> >&>(*SM), row, col, Rowdim, Coldim),
 			f (SM ->  field()), vd(SM -> field())
 		{
-		}
+			}
 
 		/** Constructor from an existing submatrix and dimensions
 		 * @param SM reference to Submatrix from which to
@@ -345,7 +346,7 @@ namespace LinBox
 			   size_t col,
 			   size_t Rowdim,
 			   size_t Coldim ) :
-			BlasSubmatrix<Field> (const_cast<Submatrix<BlasMatrix<Field> >&>(SM), row, col, Rowdim, Coldim),
+			BlasSubmatrix<Matrix> (const_cast<Submatrix<BlasMatrix<Field> >&>(SM), row, col, Rowdim, Coldim),
 			f (SM. field()), vd(SM. field())
 		{ }
 
@@ -360,7 +361,7 @@ namespace LinBox
 		std::istream& read (std::istream& is)
 		{
 
-			BlasSubmatrix<Field>::read (is, f);
+			BlasSubmatrix<Matrix>::read (is, f);
 
 			return is;
 		}
@@ -369,7 +370,7 @@ namespace LinBox
 		std::ostream& write (std::ostream& os) const
 		{
 
-			BlasSubmatrix<Field>::write (os, f);
+			BlasSubmatrix<Matrix>::write (os, f);
 
 			return os;
 		}
@@ -385,7 +386,7 @@ namespace LinBox
 		Vect1 &apply (Vect1 &y, const Vect2 &x) const
 		{
 
-			typename BlasSubmatrix<Field>::ConstRowIterator p;
+			typename BlasSubmatrix<Matrix>::ConstRowIterator p;
 
 			typename Vect1::iterator p_y = y.begin ();
 
@@ -407,7 +408,7 @@ namespace LinBox
 		Vect1 &applyTranspose (Vect1 &y, const Vect2 &x) const
 		{
 
-			typename BlasSubmatrix<Field>::ConstColIterator colp;
+			typename BlasSubmatrix<Matrix>::ConstColIterator colp;
 
 			typename Vect1::iterator p_y = y.begin ();
 
@@ -486,12 +487,12 @@ namespace LinBox
 		template<class OutVector, class InVector>
 		OutVector& apply (OutVector &y, const InVector& x) const
 		{
-			std::fill (_z.begin (), _z.begin () + _col, _BB_data.field().zero);
-			std::fill (_z.begin () + _col + _coldim, _z.end (), _BB_data.field().zero);
+			std::fill (_z.begin (), _z.begin () + (ptrdiff_t)_col, _BB_data.field().zero);
+			std::fill (_z.begin () + (ptrdiff_t)(_col + _coldim), _z.end (), _BB_data.field().zero);
 
-			copy (x.begin (), x.end (), _z.begin () + _col);  // Copying. Yuck.
+			copy (x.begin (), x.end (), _z.begin () +(ptrdiff_t) _col);  // Copying. Yuck.
 			_BB_data.apply (_y, _z);
-			copy (_y.begin () + _row, _y.begin () + _row + _rowdim, y.begin ());
+			copy (_y.begin () +(ptrdiff_t) _row, _y.begin () +(ptrdiff_t) (_row + _rowdim), y.begin ());
 			return y;
 		}
 
@@ -507,12 +508,12 @@ namespace LinBox
 		template<class OutVector, class InVector>
 		OutVector& applyTranspose (OutVector &y, const InVector& x) const
 		{
-			std::fill (_y.begin (), _y.begin () + _row, _BB_data.field().zero);
-			std::fill (_y.begin () + _row + _rowdim, _y.end (), _BB_data.field().zero);
+			std::fill (_y.begin (), _y.begin () +(ptrdiff_t) _row, _BB_data.field().zero);
+			std::fill (_y.begin () +(ptrdiff_t)( _row + _rowdim), _y.end (), _BB_data.field().zero);
 
-			copy (x.begin (), x.end (), _y.begin () + _row);  // Copying. Yuck.
+			copy (x.begin (), x.end (), _y.begin () + (ptrdiff_t)_row);  // Copying. Yuck.
 			_BB_data.applyTranspose (_z, _y);
-			copy (_z.begin () + _col, _z.begin () + _col + _coldim, y.begin ());
+			copy (_z.begin () + (ptrdiff_t)_col, _z.begin () + (ptrdiff_t)(_col + _coldim), y.begin ());
 			return y;
 		}
 
@@ -619,4 +620,3 @@ namespace LinBox
 // c-basic-offset: 8
 // End:
 // vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,\:0,t0,+0,=s
-
