@@ -64,7 +64,7 @@ namespace LinBox
 			_field (F), _size (size), _seed (seed)
 		{
 			if (seed == 0)
-				_seed = time (NULL);
+				_seed = uint32_t(time (NULL));
 		}
 
 
@@ -98,18 +98,19 @@ namespace LinBox
 		 */
 		Element &random (Element &a)  const
 		{
-			int value = 0;
 
 			if (_size == 0) {
-			unsigned int s;
+				unsigned int s;
 				s = _seed;
 
-				value = rand_r (&s);
+				//(rand_r is obsolete)
+				 srand (s);
+				 int value = rand();
 
 				mpz_set_si (mpq_numref (a.rep), value);
 
 				do {
-					value = rand_r (&s);
+					value = rand();
 				} while (value == 0);
 
 				const_cast<integer&>(_seed) = s;
@@ -120,7 +121,8 @@ namespace LinBox
 				int num, den;
 
 				s = _seed;
-				num = rand_r (&s);
+				srand (s);
+				num =  rand();
 
 				if (_size > 0) {
 					int tmp = (int)_size;
@@ -128,7 +130,8 @@ namespace LinBox
 					den = 1L;
 				}
 				else {
-					den = rand_r (&s);
+					srand (s);
+					den =  rand();
 				}
 
 				const_cast<integer&>(_seed) = s;
@@ -156,6 +159,10 @@ namespace LinBox
 			random (tmp);
 			return (a = ElementEnvelope <GMPRationalField> (tmp));
 		}
+        
+        const GMPRationalField& ring() const { return _field; }
+        
+                
 
 	private:
 
@@ -170,11 +177,10 @@ namespace LinBox
 #endif // __LINBOX_randiter_gmp_random_H
 
 
-// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,:0,t0,+0,=s
 // Local Variables:
 // mode: C++
 // tab-width: 8
 // indent-tabs-mode: nil
 // c-basic-offset: 8
 // End:
-
+// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,\:0,t0,+0,=s
