@@ -5,20 +5,20 @@
  *
  * ------------------------------------
  *
- * 
+ *
  * ========LICENCE========
  * This file is part of the library LinBox.
- * 
+ *
  * LinBox is free software: you can redistribute it and/or modify
  * it under the terms of the  GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
- * 
+ *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
@@ -40,22 +40,22 @@ namespace LinBox
 	class MultiModRandomPrime {
 	protected:
 
-		size_t   _bits;
+		uint64_t   _bits;
 		size_t   _size;
 		integer _shift;
 
 	public:
 
 
-		MultiModRandomPrime(size_t n=1, size_t bits = 30, unsigned long seed = 0) :
+		MultiModRandomPrime(size_t n=1, uint64_t bits = 30, unsigned long seed = 0) :
 			_bits(bits), _size(n)
 		{
 			_shift = integer(1)<<_bits;
 
 			if (! seed)
-				MultiModRandomPrime::setSeed( (unsigned long) BaseTimer::seed() );
+				setSeed( (unsigned long) BaseTimer::seed() );
 			else
-				MultiModRandomPrime::setSeed( seed );
+				setSeed( seed );
 		}
 
 		// define the prime type
@@ -68,12 +68,13 @@ namespace LinBox
 		{
 			std::cout<<"ist used\n";
 			std::vector<integer> tmp(_size);
+			Givaro::IntPrimeDom IPD;
 			for (size_t i=0;i<_size; ++i){
 				do {
 					integer::random(tmp[i],_bits-1);
 					tmp[i]= _shift-tmp[i];
-					nextprime(tmp[i],tmp[i]);
-				}
+					IPD.nextprimein(tmp[i]);
+			}
 				while (std::find(tmp.begin(), tmp.begin()+(long)i, tmp[i]) != (tmp.begin()+(long)i ));
 			}
 			return tmp;
@@ -84,12 +85,13 @@ namespace LinBox
 		 */
 		inline Prime_Type randomPrime (Prime_Type& p) const
 		{
+			Givaro::IntPrimeDom IPD;
 			for (size_t i=0;i<p.size(); ++i){
 				do {
 					integer::random(p[i],_bits-1);
 					p[i]= _shift-p[i];
-					nextprime(p[i],p[i]);
-				}
+					IPD.nextprimein(p[i]);
+			}
 				while (std::find(p.begin(), p.begin()+(long)(i-1), p[i]) != (p.begin()+(long)(i-1) ));
 			}
 			return p;
@@ -109,9 +111,9 @@ namespace LinBox
 		{
 			Prime_Type tmp;
 			integer acc=1, cur=maxPrime;
-
+			Givaro::IntPrimeDom IPD;
 			do {
-				prevprime(cur, cur);
+				IPD.prevprimein(cur);
 				tmp.push_back(cur);
 				acc*=cur;
 			} while (acc < productBound);
