@@ -35,9 +35,9 @@
 #include <iostream>
 #include <givaro/givpoly1.h>
 #include <givaro/givpoly1factor.h>
+#include <givaro/zring.h>
+
 #include "linbox/integer.h"
-#include "linbox/field/unparametric.h"
-#include "linbox/field/givaro.h"
 //#include "linbox/element/givaro-polynomial.h"
 
 
@@ -53,26 +53,28 @@ namespace LinBox
 	 *  @tparam StorageTag
 	 */
 	template <class Domain, class StorageTag= Givaro::Dense>
-	class GivPolynomialRing : public Givaro::Poly1FactorDom< GivaroField<Domain>,StorageTag> {
+	class GivPolynomialRing : public Givaro::Poly1FactorDom< Domain,StorageTag> {
 	public:
 
 		//	using Givaro::Poly1FactorDom<Domain,StorageTag>::eval;
-        typedef typename Givaro::Poly1FactorDom<GivaroField<Domain>,StorageTag> Father_t;
+		typedef typename Givaro::Poly1FactorDom<Domain,StorageTag> Father_t;
 		typedef typename Father_t::Element Element;
 		typedef Element Polynomial;
 
-		GivPolynomialRing () {}
+		typedef Father_t Rep;
 
-		GivPolynomialRing (const Domain& D) : Father_t( GivaroField<Domain>(D) )
+		// GivPolynomialRing () {}
+
+		GivPolynomialRing (const Domain& D) : Father_t( Domain(D) )
 		{}
 
 		GivPolynomialRing (const Domain& D, const Givaro::Indeter& I) :
-                Father_t(GivaroField<Domain>(D), I)
+                Father_t(Domain(D), I)
 		{}
 
 		template<class PolyCont>
 		PolyCont& factor (PolyCont& factors,
-				  std::vector<unsigned long>& exp,
+				  std::vector<uint64_t>& exp,
 				  const Polynomial& P)
             {
 
@@ -80,9 +82,9 @@ namespace LinBox
                     // at least without pointers ...
                 std::vector<Polynomial> Lf;
                 CZfactor(Lf, exp, P); // Cantor-Zassenhaus factorization
-                factors.resize(Lf.size());
+                factors.resize((size_t)Lf.size());
                 for(size_t i=0;  i<Lf.size(); ++i)
-                    factors[i] = new Polynomial(Lf[i]);
+                    factors[(size_t)i] = new Polynomial(Lf[(size_t)i]);
 
                 return factors;
             }
@@ -93,23 +95,24 @@ namespace LinBox
 
 #ifdef __LINBOX_HAVE_NTL
 }
-#include "linbox/field/ntl.h"
+
+#include "linbox/ring/ntl.h"
 #include "NTL/ZZXFactoring.h"
 namespace LinBox
 {
-	typedef GivPolynomialRing<UnparametricField<integer>, Givaro::Dense> GivPolIntDense;
+	typedef GivPolynomialRing<Givaro::ZRing<integer>, Givaro::Dense> GivPolIntDense;
 
 	template <>
 	template <>
 	std::vector<GivPolIntDense::Element* >&
 	GivPolIntDense::factor (std::vector<GivPolIntDense::Element* >& factors,
-				std::vector<unsigned long>& exp,
+				std::vector<uint64_t>& exp,
 				const GivPolIntDense::Element &P)
 	{
 		NTL::ZZXFac_InitNumPrimes = 1;
 		NTL::ZZX f;
 		for (size_t i = 0; i < P.size(); ++i){
-			NTL::SetCoeff (f, i, NTL::to_ZZ((std::string( P[i] )).c_str()) );
+			NTL::SetCoeff (f, (long)i, NTL::to_ZZ((std::string( P[(size_t)i] )).c_str()) );
 		}
 		NTL::vec_pair_ZZX_long ntlfactors;
 		NTL::ZZ c;
@@ -117,54 +120,54 @@ namespace LinBox
 
 		NTL::ZZ t;
 		NTL_ZZ NTLIntDom;
-		factors.resize(ntlfactors.length());
-		exp.resize(ntlfactors.length());
+		factors.resize((size_t)ntlfactors.length());
+		exp.resize((size_t)ntlfactors.length());
 		for (int i= 0; i<ntlfactors.length(); ++i) {
-			factors[i] = new GivPolIntDense::Element( deg(ntlfactors[i].a)+1 );
+			factors[(size_t)i] = new GivPolIntDense::Element( (size_t)( deg(ntlfactors[i].a)+1 ));
 			for(int j = 0; j <= deg(ntlfactors[i].a); ++j) {
 				NTL::GetCoeff(t,ntlfactors[i].a,j);
-				NTLIntDom.convert( factors[i]->operator[](j), t );
+				NTLIntDom.convert( factors[(size_t)i]->operator[]((size_t)j), t );
 			}
-			exp[i] = ntlfactors[i].b;
+			exp[(size_t)i] =(unsigned long) ntlfactors[i].b;
 		}
 		return factors;
 	}
 }
 
-#include "linbox/field/PID-integer.h"
-namespace LinBox
-{
-	typedef GivPolynomialRing<PID_integer, Givaro::Dense> GivPolPIDIntDense;
-	template <>
-	template <>
-	std::vector<GivPolPIDIntDense::Element* >&
-	GivPolPIDIntDense::factor<std::vector<GivPolPIDIntDense::Element* > > (std::vector<GivPolPIDIntDense::Element* >& factors,
-			std::vector<unsigned long>& exp,
-			const GivPolPIDIntDense::Element &P)
-	{
-		NTL::ZZXFac_InitNumPrimes = 1;
-		NTL::ZZX f;
-		for (size_t i = 0; i < P.size(); ++i){
-			NTL::SetCoeff (f, i, NTL::to_ZZ((std::string( P[i] )).c_str()) );
-		}
-		NTL::vec_pair_ZZX_long ntlfactors;
-		NTL::ZZ c;
-		NTL::factor (c, ntlfactors, f);
+// #include "givaro/zring.h"
+ namespace LinBox
+ {
+// 	typedef GivPolynomialRing<Givaro::ZRing<Integer>, Givaro::Dense> GivPolPIDIntDense;
+// 	template <>
+// 	template <>
+// 	std::vector<GivPolPIDIntDense::Element* >&
+// 	GivPolPIDIntDense::factor<std::vector<GivPolPIDIntDense::Element* > > (std::vector<GivPolPIDIntDense::Element* >& factors,
+// 			std::vector<unsigned long>& exp,
+// 			const GivPolPIDIntDense::Element &P)
+// 	{
+// 		NTL::ZZXFac_InitNumPrimes = 1;
+// 		NTL::ZZX f;
+// 		for (size_t i = 0; i < P.size(); ++i){
+// 			NTL::SetCoeff (f, (long)i, NTL::to_ZZ((std::string( P[(size_t)i] )).c_str()) );
+// 		}
+// 		NTL::vec_pair_ZZX_long ntlfactors;
+// 		NTL::ZZ c;
+// 		NTL::factor (c, ntlfactors, f);
 
-		NTL::ZZ t;
-		NTL_ZZ NTLIntDom;
-		factors.resize(ntlfactors.length());
-		exp.resize(ntlfactors.length());
-		for (int i= 0; i<ntlfactors.length(); ++i) {
-			factors[i] = new GivPolPIDIntDense::Element( deg(ntlfactors[i].a)+1 );
-			for(int j = 0; j <= deg(ntlfactors[i].a); ++j) {
-				NTL::GetCoeff(t,ntlfactors[i].a,j);
-				NTLIntDom.convert( factors[i]->operator[](j), t );
-			}
-			exp[i] = ntlfactors[i].b;
-		}
-		return factors;
-	}
+// 		NTL::ZZ t;
+// 		NTL_ZZ NTLIntDom;
+// 		factors.resize((size_t)ntlfactors.length());
+// 		exp.resize((size_t)ntlfactors.length());
+// 		for (int i= 0; i<ntlfactors.length(); ++i) {
+// 			factors[(size_t)i] = new GivPolPIDIntDense::Element( (size_t)deg(ntlfactors[i].a)+1 );
+// 			for(int j = 0; j <= deg(ntlfactors[i].a); ++j) {
+// 				NTL::GetCoeff(t,ntlfactors[i].a,j);
+// 				NTLIntDom.convert( factors[(size_t)i]->operator[]((size_t)j), t );
+// 			}
+// 			exp[(size_t)i] = (unsigned long)ntlfactors[i].b;
+// 		}
+// 		return factors;
+// 	}
 
 	typedef GivPolynomialRing< NTL_ZZ , Givaro::Dense> GivPolZZDense;
 	template <>
@@ -172,46 +175,46 @@ namespace LinBox
 	std::vector<GivPolZZDense::Element* >&
 	GivPolZZDense::factor<std::vector<GivPolZZDense::Element* > >
 	(std::vector<GivPolZZDense::Element* >& factors,
-	 std::vector<unsigned long>& exp,
+	 std::vector<uint64_t>& exp,
 	 const GivPolZZDense::Element &P)
 	{
 		NTL::ZZXFac_InitNumPrimes = 1;
 		NTL::ZZX f;
 		for (size_t i = 0; i < P.size(); ++i){
-			NTL::SetCoeff (f, i, P[i]);
+			NTL::SetCoeff (f, (long)i, P[(size_t)i]);
 		}
 		NTL::vec_pair_ZZX_long ntlfactors;
 		NTL::ZZ c;
 		NTL::factor (c, ntlfactors, f);
 
 		NTL::ZZ t;
-		factors.resize(ntlfactors.length());
-		exp.resize(ntlfactors.length());
+		factors.resize((size_t)ntlfactors.length());
+		exp.resize((size_t)ntlfactors.length());
 		for (int i= 0; i<ntlfactors.length(); ++i) {
-			factors[i] = new GivPolZZDense::Element( deg(ntlfactors[i].a)+1 );
+			factors[(size_t)i] = new GivPolZZDense::Element((size_t) deg(ntlfactors[i].a)+1 );
 			for(int j = 0; j <= deg(ntlfactors[i].a); ++j) {
-				NTL::GetCoeff( factors[i]->operator[](j),ntlfactors[i].a,j);
+				NTL::GetCoeff( factors[(size_t)i]->operator[]((size_t)j),ntlfactors[i].a,j);
 			}
-			exp[i] = ntlfactors[i].b;
+			exp[(size_t)i] = (unsigned long)ntlfactors[i].b;
 		}
 		return factors;
 	}
 
 #endif
 
-	typedef GivPolynomialRing<Modular<double>, Givaro::Dense> GivPolMdDense;
+	typedef GivPolynomialRing<Givaro::Modular<double>, Givaro::Dense> GivPolMdDense;
 
 	template <>
 	template <>
 	std::vector<GivPolMdDense::Element *>&
 	GivPolMdDense::factor (std::vector<GivPolMdDense::Element* > & factors,
-			       std::vector<unsigned long>& exp,
+			       std::vector<uint64_t>& exp,
 			       const GivPolMdDense::Element& P)
 	{
 		integer charac;
 		_domain.characteristic(charac);
 		double p = charac;
-		typedef GivaroField<Modular<double> > GivModDouble;
+		typedef Givaro::Modular<double> GivModDouble;
 		typedef Givaro::Poly1FactorDom< GivModDouble, Givaro::Dense, GivModDouble::RandIter> PolysDouble;
 
 
@@ -220,7 +223,7 @@ namespace LinBox
 		PFD.CZfactor ( factors2, exp, static_cast<PolysDouble::Element>(P),p);
 
 		//std::cerr<<"factorization done"<<std::endl;
-		factors.resize(factors2.size());
+		factors.resize((size_t)factors2.size());
 		std::vector<GivPolMdDense::Element* >::iterator itf = factors.begin();
 		std::vector<PolysDouble::Element >::const_iterator itf2 = factors2.begin();
 		for (; itf2 != factors2.end();++itf,++itf2){
