@@ -25,18 +25,13 @@
 #ifndef __LINBOX_charpoly_H
 #define __LINBOX_charpoly_H
 
-
-
 #include "linbox/solutions/methods.h"
 #include "linbox/util/debug.h"
 #include "linbox/field/field-traits.h"
-#include "linbox/matrix/blas-matrix.h"
-#include "linbox/algorithms/blas-domain.h"
-
-#ifdef __LINBOX_HAVE_GIVARO
-// BBcharpoly without givaropolynomials is not yet implemented
+#include "linbox/matrix/dense-matrix.h"
+#include "linbox/matrix/matrix-domain.h"
+#include "linbox/randiter/random-prime.h"
 #include "linbox/algorithms/bbcharpoly.h"
-#endif
 // Namespace in which all LinBox library code resides
 
 namespace LinBox
@@ -90,8 +85,9 @@ namespace LinBox
 	}
 
 	// The charpoly with Hybrid Method
+	//! @bug not Hybrid at all
 	template<class Polynomial, class Blackbox>
-	Polynomial &charpoly (Polynomial                       &P,
+	Polynomial &charpoly (Polynomial            &P,
 			      const Blackbox                   & A,
 			      const RingCategories::ModularTag & tag,
 			      const Method::Hybrid             & M)
@@ -102,19 +98,23 @@ namespace LinBox
 	}
 
 	// The charpoly with Hybrid Method
+	//! @bug not Hybrid at all
 	template<class Polynomial, class Domain>
-	Polynomial &charpoly (Polynomial                       &P,
+	Polynomial &charpoly (Polynomial            &P,
 			      const SparseMatrix<Domain>       & A,
 			      const RingCategories::ModularTag & tag,
 			      const Method::Hybrid             & M)
 	{
 		// not yet a hybrid
-		return charpoly(P, A, tag, Method::Blackbox(M));
+                // bb method broken, default to dense method
+		return charpoly(P, A, tag, Method::BlasElimination(M));
+//		return charpoly(P, A, tag, Method::Blackbox(M));
 	}
 
 	// The charpoly with Hybrid Method
+	//! @bug not Hybrid at all
 	template<class Polynomial, class Domain>
-	Polynomial &charpoly (Polynomial                       &P,
+	Polynomial &charpoly (Polynomial            &P,
 			      const BlasMatrix<Domain>         & A,
 			      const RingCategories::ModularTag & tag,
 			      const Method::Hybrid             & M)
@@ -145,18 +145,39 @@ namespace LinBox
 	 * @param M
 	 */
 	template < class Polynomial, class Blackbox >
-	Polynomial& charpoly (Polynomial                       & P,
-			      const Blackbox                   & A,
-			      const RingCategories::ModularTag & tag,
-			      const Method::BlasElimination    & M)
+	BlasVector<typename Blackbox::Field,Polynomial >&
+	charpoly (BlasVector<typename Blackbox::Field,Polynomial > & P,
+		  const Blackbox                                   & A,
+		  const RingCategories::ModularTag                 & tag,
+		  const Method::BlasElimination                    & M)
 	{
 		if (A.coldim() != A.rowdim())
 			throw LinboxError("LinBox ERROR: matrix must be square for characteristic polynomial computation\n");
 
 		BlasMatrix< typename Blackbox::Field >     BBB (A);
 		BlasMatrixDomain< typename Blackbox::Field > BMD (BBB.field());
-		return BMD.charpoly (P, static_cast<BlasMatrix<typename Blackbox::Field> >(BBB));
+		BMD.charpoly (P, static_cast<BlasMatrix<typename Blackbox::Field> >(BBB));
+		return P  ;
 	}
+	template < class Polynomial, class Blackbox >
+	Polynomial& charpoly (Polynomial                       & P,
+			      const Blackbox                   & A,
+			      const RingCategories::ModularTag & tag,
+			      const Method::BlasElimination    & M)
+	{
+
+		if (A.coldim() != A.rowdim())
+			throw LinboxError("LinBox ERROR: matrix must be square for characteristic polynomial computation\n");
+
+		BlasMatrix< typename Blackbox::Field >     BBB (A);
+		BlasMatrixDomain< typename Blackbox::Field > BMD (BBB.field());
+		BlasVector<typename Blackbox::Field,Polynomial> P2(A.field(),P);
+		BMD.charpoly (P2, static_cast<BlasMatrix<typename Blackbox::Field> >(BBB));
+		return P = P2.getRep() ;
+
+
+	}
+
 
 }
 
@@ -182,7 +203,9 @@ namespace LinBox
 		{
 			typedef typename Blackbox::template rebind<Field>::other FBlackbox;
 			FBlackbox Ap(A, F);
-			return charpoly( P, Ap, typename FieldTraits<Field>::categoryTag(), M);
+
+                        return charpoly( P, Ap, typename FieldTraits<Field>::categoryTag(), M);
+                        // std::cerr << "Charpoly(A) mod "<<F.characteristic()<<" = "<<P;
 			// 			integer p;
 			// 			F.characteristic(p);
 			//			std::cerr<<"Charpoly(A) mod "<<p<<" = "<<P;
@@ -196,18 +219,20 @@ namespace LinBox
 			      const Method::Hybrid	       & M)
 	{
 		commentator().start ("Integer Charpoly", "Icharpoly");
-		if ( (A.rowdim() < 1000) && (A.coldim() <1000) )
+                    // bb method broken, default to dense method
+		if (1/* (A.rowdim() < 1000) && (A.coldim() <1000) */)
 			charpoly(P, A, tag, Method::BlasElimination(M) );
-		else
+                else
 			charpoly(P, A, tag, Method::Blackbox(M) );
 		commentator().stop ("done", NULL, "Icharpoly");
 		return P;
 	}
 
-	//#if 0
-#if defined(__LINBOX_HAVE_NTL) && defined(__LINBOX_HAVE_GIVARO)
 }
 
+#if 0 // CIA is buggy (try with matrix(ZZ,4,[1..16]) )
+//#if defined(__LINBOX_HAVE_NTL)
+
 #include "linbox/algorithms/cia.h"
 namespace LinBox
 {
@@ -221,7 +246,9 @@ namespace LinBox
 			      const Method::Hybrid              &M)
 	{
 		// not yet a hybrid
-		return charpoly(P, A, tag, Method::Blackbox(M));
+                    // bb method broken, default to dense method
+                return charpoly(P, A, tag, Method::BlasElimination(M));
+//		return charpoly(P, A, tag, Method::Blackbox(M));
 	}
 #endif
 
@@ -270,7 +297,7 @@ namespace LinBox
 	{
 		if (A.coldim() != A.rowdim())
 			throw LinboxError("LinBox ERROR: matrix must be square for characteristic polynomial computation\n");
-		typename GivPolynomialRing<typename Blackbox::Field>::Element Pg;
+		typename Givaro::Poly1Dom<typename Blackbox::Field, Givaro::Dense>::Element Pg;
 		return P = cia (Pg, A, M);
 	}
 
@@ -290,19 +317,21 @@ namespace LinBox
 	{
 		if (A.coldim() != A.rowdim())
 			throw LinboxError("LinBox ERROR: matrix must be square for characteristic polynomial computation\n");
-		typename GivPolynomialRing<typename Blackbox::Field>::Element Pg;
-		return P = BBcharpoly::blackboxcharpoly (Pg, A, tag, M);
+// 		typename Givaro::Poly1Dom<typename Blackbox::Field>::Element Pg;
+// 		return P = BBcharpoly::blackboxcharpoly (Pg, A, tag, M);
+                    // BB method broken: default to dense method
+                return charpoly(P, A, tag, Method::BlasElimination(M) );
+                    //return BBcharpoly::blackboxcharpoly (P, A, tag, M);
 	}
 
-
-#else //  no NTL or no Givaro (??)
 }
 
-#include "linbox/field/modular.h"
+#else //  no NTL
+
+#include "linbox/ring/modular.h"
 #include "linbox/algorithms/cra-domain.h"
 #include "linbox/algorithms/cra-full-multip.h"
 #include "linbox/algorithms/cra-early-multip.h"
-#include "linbox/randiter/random-prime.h"
 #include "linbox/algorithms/matrix-hom.h"
 
 namespace LinBox
@@ -369,9 +398,9 @@ namespace LinBox
 		size_t n=A.coldim();
 		double hadamarcp = n/2.0*(log(double(n))+2*log(double(max))+0.21163275)/log(2.0);
 
-		ChineseRemainder< FullMultipCRA<Modular<double> > > cra(hadamarcp);
+		ChineseRemainder< FullMultipCRA<Givaro::Modular<double> > > cra(hadamarcp);
 #endif
-		ChineseRemainder< EarlyMultipCRA<Modular<double> > > cra(3UL);
+		ChineseRemainder< EarlyMultipCRA<Givaro::Modular<double> > > cra(3UL);
 
 		IntegerModularCharpoly<Blackbox,Method::Blackbox> iteration(A, M);
 		cra.operator() (P, iteration, genprime);
@@ -410,18 +439,21 @@ namespace LinBox
 		double hadamarcp = n/2.0*(log(double(n))+2*log(double(max))+0.21163275)/log(2.0);
 
 
-		ChineseRemainder< FullMultipCRA<Modular<double> > > cra(hadamarcp);
+		ChineseRemainder< FullMultipCRA<Givaro::Modular<double> > > cra(hadamarcp);
 #endif
-		ChineseRemainder< EarlyMultipCRA<Modular<double> > > cra(3UL);
+		ChineseRemainder< EarlyMultipCRA<Givaro::Modular<double> > > cra(3UL);
 		IntegerModularCharpoly<Blackbox,Method::BlasElimination> iteration(A, M);
 		cra(P, iteration, genprime);
 		commentator().stop ("done", NULL, "IbbCharpoly");
 		return P;
 	}
 
+}
 
 #endif
 
+namespace LinBox
+{
 	/** Compute the characteristic polynomial over \f$\mathbf{Z}_p\f$.
 	 *
 	 * Compute the characteristic polynomial of a matrix, represented via
@@ -441,12 +473,9 @@ namespace LinBox
 		if (A.coldim() != A.rowdim())
 			throw LinboxError("LinBox ERROR: matrix must be square for characteristic polynomial computation\n");
 
-#ifdef __LINBOX_HAVE_GIVARO
-		typename GivPolynomialRing<typename Blackbox::Field>::Element Pg;
-		return P = BBcharpoly::blackboxcharpoly (Pg, A, tag, M);
-#else
-		return charpoly(P, A, tag, Method::BlasElimination());
-#endif
+// 		typename Givaro::Poly1Dom<typename Blackbox::Field>::Element Pg;
+// 		return P = BBcharpoly::blackboxcharpoly (Pg, A, tag, M);
+		return BBcharpoly::blackboxcharpoly (P, A, tag, M);
 	}
 
 	template < class Blackbox, class Polynomial, class MyMethod>
@@ -455,11 +484,12 @@ namespace LinBox
 	{
 		commentator().start ("Rational Charpoly", "Rcharpoly");
 
-		RandomPrimeIterator genprime( 26-(int)ceil(log((double)A.rowdim())*0.7213475205));
-		RationalRemainder2< VarPrecEarlyMultipCRA<Modular<double> > > rra(3UL);
+		RandomPrimeIterator genprime( 26-(unsigned int)ceil(log((double)A.rowdim())*0.7213475205));
+		RationalRemainder2< VarPrecEarlyMultipCRA<Givaro::Modular<double> > > rra(3UL);
 		IntegerModularCharpoly<Blackbox,MyMethod> iteration(A, M);
 
-		std::vector<Integer> PP; // use of integer due to non genericity of cra. PG 2005-08-04
+		Givaro::ZRing<Integer> Z;
+		BlasVector<Givaro::ZRing<Integer> > PP(Z); // use of integer due to non genericity of cra. PG 2005-08-04
 		Integer den;
 		rra(PP,den, iteration, genprime);
 		size_t i =0;
@@ -476,11 +506,10 @@ namespace LinBox
 #endif // __LINBOX_charpoly_H
 
 
-// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,:0,t0,+0,=s
 // Local Variables:
 // mode: C++
 // tab-width: 8
 // indent-tabs-mode: nil
 // c-basic-offset: 8
 // End:
-
+// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,\:0,t0,+0,=s
