@@ -47,7 +47,8 @@
 #include <fstream>
 #include <cstring>
 
-#include "linbox/util/timer.h"
+//#include "linbox/util/timer.h"
+#include "givaro/givtimer.h"
 
 #ifndef MAX
 #  define MAX(a,b) (((a) > (b)) ? (a) : (b))
@@ -186,6 +187,10 @@ namespace LinBox
 		void start (const char *description,
 			    const char *fn = (const char *) 0,
 			    unsigned long len = 0);
+		void start (std::string description,
+			    const char *fn = (const char *) 0,
+			    unsigned long len = 0);
+
 
 		/** @internal
 		 * Start a new iteration.
@@ -518,6 +523,11 @@ namespace LinBox
 			report ((MessageLevel) msglevel, msgclass) << msg << std::endl;
 		}
 
+		void start (std::string id, const char *msg, long msglevel, const char *msgclass)
+		{
+			start(id.c_str(),msg,msglevel,msgclass);
+		}
+
 		/** @internal
 		 * Stop an activity.
 		 * @param msg Message to print
@@ -624,7 +634,7 @@ namespace LinBox
 			const char              *_fn;
 			unsigned long            _len;
 			unsigned long            _progress;
-		 Givaro::Timer                    _timer;
+		 Givaro::RealTimer                    _timer;
 			Estimator                _estimate;
 		};
 
@@ -838,11 +848,11 @@ namespace LinBox
 		inline void setPrintParameters (unsigned long, unsigned long , const char *)
 	       	{}
 		inline bool isPrinted (unsigned long , unsigned long , const char * = (const char *) 0)
-		{
-		       	return false;
-		}
+		{ return false; }
 	};
 
+	class ActivityState {};
+
 	class Commentator {
 	public:
 #if 0
@@ -874,7 +884,7 @@ namespace LinBox
 			LEVEL_UNIMPORTANT  =  3
 		};
 
-		inline std::ostream &report (long , const char *)
+		inline std::ostream &report (long level = LEVEL_IMPORTANT, const char *msg_class = INTERNAL_DESCRIPTION)
 		{
 		       	return cnull;
 		}
@@ -903,37 +913,23 @@ namespace LinBox
 		inline void setMaxDetailLevel (long )
 		{}
 		inline MessageClass &registerMessageClass (const char *, std::ostream &, unsigned long = 1, unsigned long = 2)
-		{
-		       	return _msgcls;
-		}
+		{ return _msgcls; }
 		inline MessageClass &cloneMessageClass (const char *, const char *)
-		{
-		       	return _msgcls;
-	       	}
+		{ return _msgcls; }
 		inline MessageClass &cloneMessageClass (const char *, const char *, std::ostream &)
-		{
-		       	return _msgcls;
-	       	}
+		{ return _msgcls; }
 		inline MessageClass &getMessageClass (const char *)
-		{
-		       	return _msgcls;
-	       	}
+		{ return _msgcls; }
 		inline void setPrintParameters (unsigned long , unsigned long , const char * = (const char *) 0)
 	       	{}
 		inline void setBriefReportParameters (OutputFormat , bool , bool , bool )
 	       	{}
 		inline bool isPrinted (unsigned long , unsigned long , const char *, const char * = (const char *) 0)
-		{
-		       	return false;
-	       	}
+		{ return false; }
 		inline bool isPrinted (unsigned long , const char *, const char * = (const char *) 0)
-		{
-		       	return false;
-		}
+		{ return false; }
 		inline bool isNullStream (const std::ostream &)
-		{
-		       	return true;
-		}
+		{ return true; }
 		inline void setBriefReportStream (std::ostream &)
 		{}
 		inline void setReportStream (std::ostream &)
@@ -951,9 +947,12 @@ namespace LinBox
 		inline void report (const char *, long , const char *)
 	       	{}
 		inline bool printed (long , const char *)
-		{
-		       	return false;
-		}
+		{ return false; }
+
+		ActivityState saveActivityState () const
+		{ return ActivityState(); }
+		void restoreActivityState (ActivityState state)
+		{}
 
 		std::ofstream cnull;
 
@@ -984,7 +983,7 @@ namespace LinBox
 #endif
 
 		MessageClass _msgcls;
-	};
+	}; // class (disabled) Commentator
 
 // 	// Default global commentator
 // 	extern Commentator commentator;
@@ -993,24 +992,24 @@ namespace LinBox
 
 #endif // DISABLE_COMMENTATOR
 
-namespace LinBox 
+namespace LinBox
 {
 	// Default static commentator
     Commentator& commentator() {
         static Commentator internal_static_commentator;
         return internal_static_commentator;
-    }   
+    }
     Commentator& commentator(std::ostream& stream) {
         static Commentator internal_static_commentator(stream);
         return internal_static_commentator;
-    }   
+    }
 }
 
 
 
 
-#include "fflas-ffpack/utils/args-parser.h"
-namespace LinBox 
+#include "linbox/util/args-parser.h"
+namespace LinBox
 {
     void parseArguments (int argc, char **argv, Argument *args, bool printDefaults = true) {
         for (int i = 1; i < argc; ++i) {
@@ -1021,8 +1020,8 @@ namespace LinBox
                 } else {
                         // Skip the argument next to "-xxx"
                         // except if next argument is a switch
-                    if ( ((i+1) < argc) && 
-                         (argv[i+1][0] != '-') ) { 
+                    if ( ((i+1) < argc) &&
+                         (argv[i+1][0] != '-') ) {
                         ++i;
                     }
                 }
@@ -1037,11 +1036,10 @@ namespace LinBox
 
 #endif // __LINBOX_commentator_H
 
-// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,:0,t0,+0,=s
 // Local Variables:
 // mode: C++
 // tab-width: 8
 // indent-tabs-mode: nil
 // c-basic-offset: 8
 // End:
-
+// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,\:0,t0,+0,=s
