@@ -53,11 +53,11 @@
 #  ifdef __GNUC__
 #    define linbox_check(check) \
         if (!(check)) \
-                 throw LinBox::PreconditionFailed (__func__, __FILE__, __LINE__, #check); //BB : should work on non gnu compilers too
+                 throw ::LinBox::PreconditionFailed (__func__, __FILE__, __LINE__, #check); //BB : should work on non gnu compilers too
 #  else
 #    define linbox_check(check) \
         if (!(check)) \
-                 throw LinBox::PreconditionFailed (__FILE__, __LINE__, #check);
+                 throw ::LinBox::PreconditionFailed (__FILE__, __LINE__, #check);
 #  endif
 #endif
 
@@ -65,7 +65,10 @@
      std::cout << "*** Warning *** " << std::endl << __func__ << " in " << __FILE__ << ':' << __LINE__ << " is not tested" << std::endl;
 
 #define THIS_CODE_MAY_NOT_COMPILE_AND_IS_NOT_TESTED \
-" *** Warning ***  this piece of code is not compiled by default and may not work"
+throw(" *** Warning ***  this piece of code is not compiled by default and may not work")
+
+#define LB_FILE_LOC \
+	__func__,__FILE__,__LINE__
 
 namespace LinBox
 { /*  Preconditions,Error,Failure,NotImplementedYet */
@@ -164,7 +167,7 @@ namespace LinBox
 		NotImplementedYet(const char * function,
 				  const char* file,
 				  int line,
-				  const char * why='\0')
+				  const char * why="\0")
 		{
 			if (_errorStream == (std::ostream *) 0)
 				_errorStream = &std::cerr;
@@ -194,9 +197,9 @@ namespace LinBox
 		 * @param what     what happened ? should not be NULL...
 		 */
 		LinBoxFailure(const char * function,
-			      const char* file,
-			      int line,
-			      const char * what='\0')
+			      const char* file="\0",
+			      int line=-1,
+			      const char * what="\0")
 		{
 			if (_errorStream == (std::ostream *) 0)
 				_errorStream = &std::cerr;
@@ -226,8 +229,8 @@ namespace LinBox
 		 * @param what     what happened ? should not be NULL...
 		 */
 		LinBoxError(const char * what,
-			    const char * function='\0',
-			    const char* file='\0',
+			    const char * function="\0",
+			    const char* file="\0",
 			    int line=-1)
 		{
 			if (_errorStream == (std::ostream *) 0)
@@ -321,7 +324,7 @@ namespace LinBox
 
 
 
-#ifdef LinBoxSrcOnly
+#if defined(LinBoxSrcOnly) or defined(LinBoxTestOnly)
 // for all-source compilation
 #include "linbox/util/debug.C"
 #endif
@@ -331,11 +334,10 @@ namespace LinBox
 #endif // __LINBOX_util_debug_H
 
 
-// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,:0,t0,+0,=s
 // Local Variables:
 // mode: C++
 // tab-width: 8
 // indent-tabs-mode: nil
 // c-basic-offset: 8
 // End:
-
+// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,\:0,t0,+0,=s
