@@ -11,20 +11,20 @@
  * ------------------------------------
  * fix testLinearity for rectangular matrices.  2006-02-17 Hui Wang
  *
- * 
+ *
  * ========LICENCE========
  * This file is part of the library LinBox.
- * 
+ *
  * LinBox is free software: you can redistribute it and/or modify
  * it under the terms of the  GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
- * 
+ *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
@@ -42,18 +42,29 @@
 #include "linbox/util/commentator.h"
 #include "linbox/util/field-axpy.h"
 #include "linbox/vector/stream.h"
-#include "linbox/vector/vector-domain.h"
+#include "linbox/matrix/matrix-domain.h"
+#include "linbox/util/matrix-stream.h"
 
 #include "test-common.h"
 
 using namespace std;
+/*
+Generic tests for black boxes
 
-// Generic tests for black boxes
-/// testBlackbox combines testTranspose and testLinearity
+For field F, BB A, vector streams s, t:
+testTranspose (F, A, s, t)
+testLinearity (A, s, t)
+testReadWrite(A)
 
-/** Random check that (u^T A) v = u^T (A v).
+testBlackboxNoRW(A) // calls testTranspose and testLinearity.
+testBlackbox(A) // calls all three generic tests.
+
+testBB(F) has been deleted. It assumed a BB could be built from a single size param (this is never true?!).
+*/
+
+/** Generic Blackbox test 1: Random check that (u^T A) v = u^T (A v).
  *
- * Take the given black box and compute u^T A v via <A^T u, v> and <u, Av> for
+ * Take the given black box and compute u^T A v via <code><A^T u, v></code> and <code><u, Av></code> for
  * randomly chosen u and v. Check whether the results are equal. In theory, this
  * should guarantee that tranpose is working correctly if apply and dot product
  * are also working correctly. Apply and dot product should, of course, be
@@ -68,14 +79,14 @@ using namespace std;
 
 template <class Field, class Blackbox, class Vector>
 static bool
-testTranspose (Field                             &F,
-	       Blackbox							 &A,
-	       LinBox::VectorStream<Vector>      &stream1,
-	       LinBox::VectorStream<Vector>      &stream2)
+testTranspose (const Field                      &F,
+	       Blackbox				&A,
+	       LinBox::VectorStream<Vector>     &stream1,
+	       LinBox::VectorStream<Vector>     &stream2)
 {
 	bool ret = true;
 
-	Vector u, v, uA, Av;
+	Vector u(F), v(F), uA(F), Av(F);
 
 	LinBox::VectorWrapper::ensureDim (u, A.rowdim ());
 	LinBox::VectorWrapper::ensureDim (uA, A.coldim ());
@@ -94,8 +105,9 @@ testTranspose (Field                             &F,
 		stream1.next (u);
 		stream2.next (v);
 
-		//ostream &report = LinBox::commentator().report (LinBox::Commentator::LEVEL_UNIMPORTANT, INTERNAL_DESCRIPTION);
+                    //ostream &report = LinBox::commentator().report (LinBox::Commentator::LEVEL_UNIMPORTANT, INTERNAL_DESCRIPTION);
 
+                    //A.write( report << "A is ");
 		VD.write( report << "Input vector u:            ", u) << endl;
 		VD.write( report << "Input vector v:            ", v) << endl;
 
@@ -104,6 +116,7 @@ testTranspose (Field                             &F,
 		VD.write( report << "Result of apply:           ", Av) << endl;
 
 		VD.dot (r1, u, Av);
+
 		A.applyTranspose (uA, u);
 
 		VD.write( report << "Result of transpose apply: ", uA) << endl;
@@ -120,8 +133,11 @@ testTranspose (Field                             &F,
 			<< "ERROR: Values are not equal" << endl;
 		}
 
-		LinBox::commentator().stop ("done");
-		LinBox::commentator().progress ();
+		if (ret)
+			LinBox::commentator().stop ("testTranspose pass");
+		else
+			LinBox::commentator().stop ("testTranspose FAIL");
+		//LinBox::commentator().progress ();
 	}
 
 	return ret;
@@ -139,21 +155,23 @@ testTranspose (Field                             &F,
  * Return true on success and false on failure
  */
 
-template <class Field, class BB, class Vector>
+template <class BB, class Vector>
 static bool
-testLinearity (Field                             &F,
+testLinearity (//const Field                             &F,
 	       BB 				 &A,
 	       LinBox::VectorStream<Vector>      &stream1,
 	       LinBox::VectorStream<Vector>      &stream2)
 {
-	bool ret = true, iter_passed;
+	bool ret = true;
 
 	size_t n = A.rowdim ();
 	size_t m = A.coldim ();
 
-	Vector x, y, xpay, Axpay, Ax, Ay, AxpaAy;
-	LinBox::VectorDomain <Field> VD (F);
-	typename Field::RandIter r (F);
+	typedef typename BB::Field Field;
+	Field F( A.field() );
+	Vector x(F), y(F), xpay(F), Axpay(F), Ax(F), Ay(F), AxpaAy(F);
+	LinBox::VectorDomain <Field> VD (A.field());
+	typename Field::RandIter r (A.field());
 	typename Field::Element alpha;
 
 	LinBox::VectorWrapper::ensureDim (x, m);
@@ -170,19 +188,18 @@ testLinearity (Field                             &F,
 	while (stream1 && stream2) {
 		LinBox::commentator().startIteration ((unsigned int) stream1.j ());
 
-		iter_passed = true;
+		bool iter_passed = true;
 
 		stream1.next (x);
 		stream2.next (y);
 
 		r.random (alpha);
 
-		//ostream &report = LinBox::commentator().report (LinBox::Commentator::LEVEL_UNIMPORTANT, INTERNAL_DESCRIPTION);
 		VD.write( report << "Input vector x: ", x) << endl;
 
 		VD.write( report << "Input vector y: ", y) << endl;
 
-		F.write( report << "Input alpha: ", alpha) << endl;
+		A.field().write( report << "Input alpha: ", alpha) << endl;
 
 		VD.axpy ( xpay, alpha, y, x);
 		A.apply ( Axpay, xpay);
@@ -207,39 +224,126 @@ testLinearity (Field                             &F,
 		if (!iter_passed)
 			LinBox::commentator().report (LinBox::Commentator::LEVEL_IMPORTANT, INTERNAL_ERROR)
 			<< "ERROR: Vectors are not equal" << endl;
-
-		LinBox::commentator().stop ("done");
-		LinBox::commentator().progress ();
+		if (iter_passed)
+			LinBox::commentator().stop ("testLinearity pass");
+		else
+			LinBox::commentator().stop ("testLinearity FAIL");
+		//LinBox::commentator().progress ();
 	}
 
 	return ret;
 }
 
-/** Generic blackbox test 4: combination of tests
+/** Generic Blackbox test 3: black box read/write.
  *
- * Call testTranspose and testLinearity.
- * If large, time apply and applyTranspose.
- * if small, call testSmallBlackbox.
+ * write the black box A, read it back and check equality.
+ *
+ * F - Field over which to perform computations
+ * A - Black box
+ *
+ * Return true on success and false on failure
  */
+
 template <class BB>
 static bool
-testBlackbox(BB &A)
+testReadWrite(BB &A)
 {
-	size_t largeThresh = 2000; // Above it do timing of apply and applyTr.
 	typedef typename BB::Field Field;
-	typedef std::vector<typename Field::Element> DenseVector;
+	bool pass = true;
+	ostream &report = LinBox::commentator().report (LinBox::Commentator::LEVEL_IMPORTANT, INTERNAL_DESCRIPTION);
+	report << "Blackbox Read/Write test: write then read back" << std::endl;
+
+	ofstream out("temp2");
+	if (not out) {
+		pass = false;
+		report << "failure to open file for writing" << std::endl;
+	}
+	A.write(out) << std::endl;
+	{
+		BB B(A.field());
+		ifstream in("temp2");
+		if (not in) {
+			pass = false;
+			report << "failure to open file for reading" << std::endl;
+		}
+		B.read(in);
+		LinBox::MatrixDomain<Field> MD(A.field());
+		if (not MD.areEqual(A, B)) {
+			pass = false;
+			report << "failure to get same matrix back from write/read" << std::endl;
+			B.write(report << "B is ") << std::endl;
+		}
+	}
+
+/*  A constructor from a matrix stream is not a required member of a blackbox. -bds
+	{
+		ifstream in("temp2");
+		if (not in) {
+			pass = false;
+			report << "failure to open file for reading" << std::endl;
+		}
+
+
+		LinBox::MatrixStream<Field> ms (A.field(), in);
+		BB B( ms );
+		LinBox::MatrixDomain<Field> MD(A.field());
+		if (not MD.areEqual(A, B)) {
+			pass = false;
+			report << "failure to get same matrix back from write/read" << std::endl;
+			B.write(report << "B is ") << std::endl;
+		}
+	}
+*/
+	if (pass) report << "PASS: successful write/read" << std::endl;
+	return pass;
+}
+
+
+template <class BB, class Vector>
+static bool
+LooksLikeZero(BB &A, LinBox::VectorStream<Vector> &stream)
+{	typedef typename BB::Field Field;
+	LinBox::VectorDomain<Field> VD(A.field());
+	LinBox::BlasVector<Field> x(A.field(), A.coldim()),
+				  y(A.field(), A.rowdim());
+	while (stream)
+	{	stream.next(x);
+		A.apply(y, x);
+		if (not VD.isZero(y)) return false;
+	}
+	ostream &report = LinBox::commentator().report (LinBox::Commentator::LEVEL_IMPORTANT, INTERNAL_DESCRIPTION);
+	report << "BB looks like zero:" << std::endl;
+	VD.write(report << "x: ", x) << std::endl;
+	VD.write(report << "Ax: ", y) << std::endl;
+	return true;
+}
+/** Generic blackbox test 3: combination of tests
+ *
+ * If large, time apply and applyTranspose.
+ * Call testTranspose and testLinearity.
+ */
+
+template <class BB>
+static bool
+testBlackboxNoRW(BB &A, bool zeroCheck=true)
+{
+	typedef typename BB::Field Field;
+	typedef LinBox::BlasVector<Field> DenseVector;
 	std::ostream &report = LinBox::commentator().report (LinBox::Commentator::LEVEL_IMPORTANT, INTERNAL_DESCRIPTION);
 	report << "testBlackbox on " << A.rowdim() << " by " << A.coldim() << " matrix." << endl;
+	//if (std::max(A.rowdim(),A.coldim())<15)
+	 //   A.write(report << "Blackbox = ");
 
 	LinBox::commentator().setMaxDepth(-1);
 	bool ret = true;
-	Field F = A.field();
+	const Field& F = A.field();
 
-	/* timing tests */   // I changed the order of all tests. Timing now is the first set of tests and then linearity and transpose
+	/* timing tests */
 	{
-		DenseVector x(A.coldim()), y(A.rowdim());
-		for(size_t i = 0; i < A.coldim(); ++i) F.init(x[i], i);
-		for(size_t i = 0; i < A.rowdim(); ++i) F.init(y[i], i);
+		size_t largeThresh = 2000; // Above it do timing of apply and applyTr.
+		DenseVector x(F,A.coldim()), y(F,A.rowdim());
+		for(size_t i = 0; i < A.coldim(); ++i) F.init(x[i], (double)i);
+		for(size_t i = 0; i < A.rowdim(); ++i) F.init(y[i], (double)i);
 		//A.apply(y, x);
 
 		if (A.coldim() >= largeThresh)
@@ -259,65 +363,44 @@ testBlackbox(BB &A)
 
 	} // timing test block
 
-#if 1
 	size_t iterations = 1;
 	typename Field::RandIter r(F);
-	LinBox::RandomDenseStream<Field, DenseVector> stream1 (F, r, A.rowdim(), iterations);
-	typename Field::Element x;
-	r.random(x);
-	LinBox::RandomDenseStream<Field, DenseVector> stream2 (F, r, A.coldim(), iterations);
-
-	LinBox::commentator().start ("\t--Testing A(ax+y) = a(Ax) + (Ay)", "testLinearity", 1);
-	ret = ret && testLinearity (F, A, stream1, stream2);
-
-	LinBox::commentator().stop (MSG_STATUS (ret),
-				  (const char *) 0, "testLinearity");
-
-	LinBox::commentator().start ("\t--Testing u^T(Av) = (u^T A)v",
-				   "testTranspose", 1);
-
-	LinBox::RandomDenseStream<Field, DenseVector> stream3 (F, r, A.rowdim(), iterations);
-	LinBox::RandomDenseStream<Field, DenseVector> stream4 (F, r, A.coldim(), iterations);
+	LinBox::RandomSparseStream<Field, DenseVector> stream1 (F, r, 1.0, A.rowdim(), iterations, 11);
+	//typename Field::Element x;
+	//r.random(x);
+	LinBox::RandomSparseStream<Field, DenseVector> stream2 (F, r, 1.0, A.coldim(), iterations);
+	ret = ret && testLinearity (A, stream1, stream2);
 
+	LinBox::RandomSparseStream<Field, DenseVector> stream3 (F, r, 1.0, A.rowdim(), iterations);
+	LinBox::RandomSparseStream<Field, DenseVector> stream4 (F, r, 1.0, A.coldim(), iterations);
 	ret = ret && testTranspose (F, A, stream3, stream4);
-	LinBox::commentator().stop (MSG_STATUS (ret),
-				  (const char *) 0, "testTranspose");
 
-#endif
+	DenseVector x(F,A.coldim()), y(F,A.rowdim());
+	if (zeroCheck) {
+		LinBox::RandomSparseStream<Field, DenseVector> stream5 (F, r, 1.0, A.coldim(), 10);
+		ret = ret & not LooksLikeZero(A, stream5);
+	}
 
 	return ret;
 }
 
-/** Generic blackbox test 5: test several sizes
- *
- * Call testTranspose and testLinearity.
- * If large, time apply and applyTranspose.
- * if small, call test
- SmallBlackbox.
- */
-template <class Field, class Blackbox>
+template <class BB>
 static bool
-testBB(Field& F)
+testBlackbox(BB &A, bool read_write=true, bool zero_check = true)
 {
-	bool pass = true;
+	bool ok = testBlackboxNoRW(A,zero_check);
+	if (ok && read_write)
+			return testReadWrite(A);
+	return ok ;
 
-	Blackbox A(10);
-	if (!testBlackbox<Field, vector<typename Field::Element> >(F, A, 1))
-		pass = false;
-	Blackbox B(10000);
-	if (!testBlackbox<Field, vector<typename Field::Element> >(F, B, 1))
-		pass = false;
-
-	return pass;
+	// return testBlackboxNoRW(A) and testReadWrite(A);
 }
-
 #endif // __LINBOX_test_blackbox_H
 
-// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,:0,t0,+0,=s
 // Local Variables:
 // mode: C++
 // tab-width: 8
 // indent-tabs-mode: nil
 // c-basic-offset: 8
 // End:
-
+// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,\:0,t0,+0,=s
