@@ -1,19 +1,19 @@
 /* linbox/tests/test-vector-domain.h
  * Copyright (C) 2001, 2002 Bradford Hovinen
- * 
+ *
  * ========LICENCE========
  * This file is part of the library LinBox.
- * 
+ *
  * LinBox is free software: you can redistribute it and/or modify
  * it under the terms of the  GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
- * 
+ *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
@@ -64,24 +64,24 @@ static bool testDotProduct (Field &F, const char *text, LinBox::VectorStream<Vec
 
 	bool ret = true;
 
-	Vector1 v1;
-	Vector2 v2;
+	Vector1 v1(F,stream1.n());
+	Vector2 v2(F,stream2.n());
 	typename Field::Element sigma, rho;
 
 	LinBox::VectorDomain<Field> VD (F);
 
 	size_t j;
 
-	LinBox::VectorWrapper::ensureDim (v1, stream1.n ());
-	LinBox::VectorWrapper::ensureDim (v2, stream2.n ());
+	// LinBox::VectorWrapper::ensureDim (v1, stream1.n ());
+	// LinBox::VectorWrapper::ensureDim (v2, stream2.n ());
 
- Givaro::Timer timer;
+	Givaro::Timer timer;
 	double totaltime = 0.0;
 
 	while (stream1 && stream2) {
 		LinBox::commentator().startIteration ((unsigned)stream1.j ());
 
-		F.init (sigma, 0);
+		F.assign (sigma, F.zero);
 
 		stream1.next (v1);
 		stream2.next (v2);
@@ -152,25 +152,24 @@ static bool testAddMul (Field &F, const char *text, LinBox::VectorStream<Vector>
 	LinBox::commentator().start (str.str ().c_str (), "testAddMul", stream1.m ());
 
 	bool ret = true;
-	bool iter_passed;
 
-	Vector v1, v2, v3, v4;
+	Vector v1(F,stream1.n()), v2(F,stream2.n()), v3(F,stream1.n()), v4(F,stream1.n());
 	typename Field::Element a;
 	typename Field::Element ainv;
 	typename Field::Element aneg;
 	typename Field::RandIter r (F);
 
-	LinBox::VectorWrapper::ensureDim (v1, stream1.n ());
-	LinBox::VectorWrapper::ensureDim (v2, stream2.n ());
-	LinBox::VectorWrapper::ensureDim (v3, stream1.n ());
-	LinBox::VectorWrapper::ensureDim (v4, stream1.n ());
+	// LinBox::VectorWrapper::ensureDim (v1, stream1.n ());
+	// LinBox::VectorWrapper::ensureDim (v2, stream2.n ());
+	// LinBox::VectorWrapper::ensureDim (v3, stream1.n ());
+	// LinBox::VectorWrapper::ensureDim (v4, stream1.n ());
 
 	LinBox::VectorDomain<Field> VD (F);
 
 	while (stream1 && stream2) {
 		LinBox::commentator().startIteration ((unsigned)stream1.j ());
 
-		iter_passed = true;
+		bool iter_passed = true;
 
 		stream1.next (v1);
 		stream2.next (v2);
@@ -255,25 +254,24 @@ static bool testSubMul (Field &F, const char *text, LinBox::VectorStream<Vector>
 	LinBox::commentator().start (str.str ().c_str (), "testSubMul", stream1.m ());
 
 	bool ret = true;
-	bool iter_passed;
 
-	Vector v1, v2, v3, v4;
+	Vector v1(F,stream1.n()), v2(F,stream2.n()), v3(F,stream1.n()), v4(F,stream1.n());
 	typename Field::Element a;
 	typename Field::Element ainv;
 	typename Field::Element aneg;
 	typename Field::RandIter r (F);
 
-	LinBox::VectorWrapper::ensureDim (v1, stream1.n ());
-	LinBox::VectorWrapper::ensureDim (v2, stream2.n ());
-	LinBox::VectorWrapper::ensureDim (v3, stream1.n ());
-	LinBox::VectorWrapper::ensureDim (v4, stream1.n ());
+	// LinBox::VectorWrapper::ensureDim (v1, stream1.n ());
+	// LinBox::VectorWrapper::ensureDim (v2, stream2.n ());
+	// LinBox::VectorWrapper::ensureDim (v3, stream1.n ());
+	// LinBox::VectorWrapper::ensureDim (v4, stream1.n ());
 
 	LinBox::VectorDomain<Field> VD (F);
 
 	while (stream1 && stream2) {
 		LinBox::commentator().startIteration ((unsigned)stream1.j ());
 
-		iter_passed = true;
+		bool iter_passed = true;
 
 		stream1.next (v1);
 		stream2.next (v2);
@@ -356,25 +354,24 @@ static bool testAXPY (Field &F, const char *text, LinBox::VectorStream<Vector> &
 	LinBox::commentator().start (str.str ().c_str (), "testAXPY", stream1.m ());
 
 	bool ret = true;
-	bool iter_passed;
 
-	Vector v1, v2, v3, v4;
+	Vector v1(F,stream1.n()), v2(F,stream2.n()), v3(F,stream1.n()), v4(F,stream1.n());
 	typename Field::Element a;
 	typename Field::Element ainv;
 	typename Field::Element aneg;
 	typename Field::RandIter r (F);
 
-	LinBox::VectorWrapper::ensureDim (v1, stream1.n ());
-	LinBox::VectorWrapper::ensureDim (v2, stream2.n ());
-	LinBox::VectorWrapper::ensureDim (v3, stream1.n ());
-	LinBox::VectorWrapper::ensureDim (v4, stream1.n ());
+	// LinBox::VectorWrapper::ensureDim (v1, stream1.n ());
+	// LinBox::VectorWrapper::ensureDim (v2, stream2.n ());
+	// LinBox::VectorWrapper::ensureDim (v3, stream1.n ());
+	// LinBox::VectorWrapper::ensureDim (v4, stream1.n ());
 
 	LinBox::VectorDomain<Field> VD (F);
 
 	while (stream1 && stream2) {
 		LinBox::commentator().startIteration ((unsigned)stream1.j ());
 
-		iter_passed = true;
+		bool iter_passed = true;
 
 		stream1.next (v1);
 		stream2.next (v2);
@@ -439,20 +436,19 @@ static bool testCopyEqual (Field &F, const char *text, LinBox::VectorStream<Vect
 	LinBox::commentator().start (str.str ().c_str (), "testCopyEqual", stream.m ());
 
 	bool ret = true;
-	bool iter_passed;
 
-	Vector1 v;
-	Vector2 w;
+	Vector1 v(F,stream.n());
+	Vector2 w(F,stream.n());
 
-	LinBox::VectorWrapper::ensureDim (v, stream.n ());
-	LinBox::VectorWrapper::ensureDim (w, stream.n ());
+	// LinBox::VectorWrapper::ensureDim (v, stream.n ());
+	// LinBox::VectorWrapper::ensureDim (w, stream.n ());
 
 	LinBox::VectorDomain<Field> VD (F);
 
 	while (stream) {
 		LinBox::commentator().startIteration ((unsigned)stream.j ());
 
-		iter_passed = true;
+		bool iter_passed = true;
 
 		stream.next (v);
 		VD.copy (w, v);
@@ -485,11 +481,10 @@ static bool testCopyEqual (Field &F, const char *text, LinBox::VectorStream<Vect
 
 #endif // __LINBOX_test_vector_domain_H
 
-// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,:0,t0,+0,=s
 // Local Variables:
 // mode: C++
 // tab-width: 8
 // indent-tabs-mode: nil
 // c-basic-offset: 8
 // End:
-
+// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,\:0,t0,+0,=s
